# -*- coding: utf-8 -*-
# vi:si:et:sw=4:sts=4:ts=4
from __future__ import print_function
from datetime import datetime
import re
import time

import ox.cache
from ox.html import decode_html, strip_tags
import ox.net


def get_news(year, month, day):
    sections = [
        'politik', 'wirtschaft', 'panorama', 'sport', 'kultur', 'netzwelt',
        'wissenschaft', 'unispiegel', 'schulspiegel', 'reise', 'auto'
    ]
    dt = datetime(year, month, day)
    day = int(dt.strftime('%j'))
    date = dt.strftime('%d.%m.%Y')
    news = []
    for section in sections:
        url = 'http://www.spiegel.de/%s/0,1518,archiv-%d-%03d,00.html' % (section, year, day)
        if date == time.strftime('%d.%m.%Y', time.localtime()):
            html = ox.net.read_url(url)
        else:
            html = ox.cache.read_url(url)
        for item in re.compile('<div class="spTeaserCenterpage(.*?)</p>', re.DOTALL).findall(html):
            dateString = strip_tags(re.compile('<div class="spDateTime">(.*?)</div>', re.DOTALL).findall(item)[0]).strip()
            try:
                description = format_string(re.compile('<p>(.*?)<', re.DOTALL).findall(item)[0])
            except:
                description = ''
            try:
                imageUrl = re.compile('<img src="(.*?)"').findall(item)[0]
            except:
                imageUrl = ''
            try:
                title = format_string(re.compile('alt=[\'|"](.*?)[\'|"] title=', re.DOTALL).findall(item)[0]).replace(' : ', ': ').replace('::', ':')
            except:
                title = ''
            if dateString[:10] == date and description and imageUrl and title.find(': ') != -1:
                new = {}
                if len(dateString) == 10:
                    new['date'] = '%s-%s-%s 00:00' % (dateString[6:10], dateString[3:5], dateString[:2])
                else:
                    new['date'] = '%s-%s-%s %s:%s' % (dateString[6:10], dateString[3:5], dateString[:2], dateString[12:14], dateString[15:17])
                # fix decode_html
                # new['description'] = format_string(decode_html(description))
                new['description'] = format_string(description)
                new['imageUrl'] = imageUrl
                new['section'] = format_section(section)
                new['title'] = format_string(title)
                new['title1'] = new['title'].replace('\xdf', '\xdf\xdf')[:len(format_string(re.compile('<h4>(.*?)</h4>', re.DOTALL).findall(item)[0]))].replace('\xdf\xdf', '\xdf')
                if new['title1'][-1:] == ':':
                    new['title1'] = new['title1'][0:-1]
                new['title2'] = new['title'][len(new['title1']) + 2:]
                new['url'] = re.compile('<a href="(.*?)"').findall(item)[0]
                if new['url'][:1] == '/':
                    new['url'] = 'http://www.spiegel.de' + new['url']
                news.append(new)
                # print('%s, %s' % (new['section'], dateString))
            '''
            elif dateString[:10] == date and not description:
                print(dateString + ' - no description')
            elif dateString[:10] == date and not imageUrl:
                print(dateString + ' - no image')
            '''
    return news

def split_title(title):
    title1 = re.compile('(.*?): ').findall(title)[0]
    title2 = re.compile(': (.*?)$').findall(title)[0]
    return [title1, title2]

def format_string(string):
    string = string.replace('<span class="spOptiBreak"> </span>', '')
    string = string.replace('\n', ' ').replace('  ', ' ').strip()
    string = string.replace('&amp;', '&').replace('&apos;', '\'').replace('&quot;', '"')
    return string

def format_section(string):
    return string[:1].upper() + string[1:].replace('spiegel', 'SPIEGEL')

def format_subsection(string):
    # SPIEGEL, SPIEGEL special
    subsection = {
        'abi': 'Abi - und dann?',
        'formel1': 'Formel 1',
        'jobundberuf': 'Job & Beruf',
        'leben': 'Leben U21',
        'mensch': 'Mensch & Technik',
        'sonst': '',
        'staedte': u'St\xc3dte',
        'ussports': 'US-Sports',
        'wunderbar': 'wunderBAR'
    }
    if string in subsection:
        return subsection[string].replace(u'\xc3', 'ae')
    return string[:1].upper() + string[1:]
        
def get_issue(year, week):
    coverUrl = 'http://www.spiegel.de/static/epaper/SP/%d/%d/ROSPANZ%d%03d0001-312.jpg' % (year, week, year, week)
    if not ox.net.exists(coverUrl):
        return None
    url = 'http://service.spiegel.de/digas/servlet/epaper?Q=SP&JG=%d&AG=%d&SE=1&AN=INHALT' % (year, week)
    contents = []
    data = ox.cache.read_url(url)
    items = re.compile('<a.?href="http://service.spiegel.de/digas/servlet/epaper\?Q=SP&JG=".?>(.*?)</a>').findall(data)
    for item in items:
        item = item[1]
        page = int(re.compile('&amp;SE=(.*?)"').findall(item)[0])
        title = strip_tags(item).strip()
        contents.append({'title': title, 'page': page})
    pageUrl = {}
    pages = page + 2
    for page in range(1, pages + 10):
        url = 'http://www.spiegel.de/static/epaper/SP/%d/%d/ROSPANZ%d%03d%04d-205.jpg' % (year, week, year, week, page)
        if ox.cache.exists(url):
            pageUrl[page] = url
        else:
            pageUrl[page] = ''
    return {'pages': pages, 'contents': contents, 'coverUrl': coverUrl, 'pageUrl': pageUrl}


def archive_issues():
    '''
    this is just an example of an archiving application
    '''
    p = {}
    import os
    from ox.utils import json
    import time
    archivePath = '/Volumes/Rolux Home/Desktop/Data/spiegel.de/Der Spiegel'
    localtime = time.localtime()
    year = int(time.strftime('%Y', localtime))
    week = int(time.strftime('%W', localtime))
    for y in range(year, 1993, -1):
        if y == year:
            wMax = week + 1
        else:
            wMax = 53
        for w in range(wMax, 0, -1):
            print('get_issue(%d, %d)' % (y, w))
            issue = get_issue(y, w)
            if issue:
                dirname = '%s/%d/%02d' % (archivePath, y, w)
                if not os.path.exists(dirname):
                    os.makedirs(dirname)
                filename = '%s/Der Spiegel %d %02d.json' % (dirname, y, w)
                if not os.path.exists(filename):
                    data = json.dumps(issue, ensure_ascii = False)
                    f = open(filename, 'w')
                    f.write(data)
                    f.close()
                filename = '%s/Der Spiegel %d %02d.txt' % (dirname, y, w)
                if not os.path.exists(filename):
                    data = []
                    for item in issue['contents']:
                        data.append('%3d %s' % (item['page'], item['title']))
                    data = '\n'.join(data)
                    f = open(filename, 'w')
                    f.write(data)
                    f.close()
                filename = '%s/Der Spiegel %d %02d.jpg' % (dirname, y, w)
                if not os.path.exists(filename):
                    data = ox.cache.read_url(issue['coverUrl'])
                    f = open(filename, 'w')
                    f.write(data)
                    f.close()
                for page in issue['pageUrl']:
                    url = issue['pageUrl'][page]
                    if url:
                        filename = '%s/Der Spiegel %d %02d %03d.jpg' % (dirname, y, w, page)
                        if not os.path.exists(filename):
                            data = ox.cache.read_url(url)
                            f = open(filename, 'w')
                            f.write(data)
                            f.close()
                if not p:
                    p = {'num': 1, 'sum': issue['pages'], 'min': issue['pages'], 'max': issue['pages']}
                else:
                    p['num'] += 1
                    p['sum'] += issue['pages']
                    if issue['pages'] < p['min']:
                        p['min'] = issue['pages']
                    if issue['pages'] > p['max']:
                        p['max'] = issue['pages']
                print(p['min'], p['sum'] / p['num'], p['max'])
            

def archive_news():
    '''
    this is just an example of an archiving application
    '''
    import os
    from ox.utils import json
    import time

    count = {}
    colon = []

    archivePath = '/Volumes/Rolux Home/Desktop/Data/spiegel.de/Spiegel Online'
    days = [0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31]
    localtime = time.localtime()
    year = int(time.strftime('%Y', localtime))
    month = int(time.strftime('%m', localtime))
    day = int(time.strftime('%d', localtime)) - 1
    for y in range(year, 1999, -1):
        if y == year:
            mMax = month
        else:
            mMax = 12
        for m in range(mMax, 0, -1):
            if y == year and m == month:
                dMax = day
            elif m == 2 and y % 4 == 0 and y % 400 != 0:
                dMax = days[m] + 1
            else:
                dMax = days[m]
            for d in range(dMax, 0, -1):
                print('get_news(%d, %d, %d)' % (y, m, d))
                news = get_news(y, m, d)
                for new in news:
                    dirname = archivePath + '/' + new['date'][0:4] + '/' + new['date'][5:7] + new['date'][8:10] + '/' + new['date'][11:13] + new['date'][14:16]
                    if not os.path.exists(dirname):
                        os.makedirs(dirname)
                    if new['url'][-5:] == '.html':
                        filename = dirname + '/' + new['url'].split('/')[-1][:-5] + '.json'
                    else:
                        filename = dirname + '/' + new['url'] + '.json'
                    if not os.path.exists(filename) or True:
                        data = json.dumps(new, ensure_ascii=False)
                        f = open(filename, 'w')
                        f.write(data)
                        f.close()
                    filename = filename[:-5] + '.txt'
                    if not os.path.exists(filename) or True:
                        data = split_title(new['title'])
                        data.append(new['description'])
                        data = '\n'.join(data)
                        f = open(filename, 'w')
                        f.write(data)
                        f.close()
                    filename = dirname + '/' + new['imageUrl'].split('/')[-1]
                    if not os.path.exists(filename):
                        data = ox.cache.read_url(new['imageUrl'])
                        f = open(filename, 'w')
                        f.write(data)
                        f.close()

                    strings = new['url'].split('/')
                    string = strings[3]
                    if len(strings) == 6:
                        string += '/' + strings[4]
                    if string not in count:
                        count[string] = {'count': 1, 'string': '%s %s http://www.spiegel.de/%s/0,1518,archiv-%d-%03d,00.html' % (new['date'], new['date'], new['section'].lower(), y, int(datetime(y, m, d).strftime('%j')))}
                    else:
                        count[string] = {'count': count[string]['count'] + 1, 'string': '%s %s' % (new['date'], count[string]['string'][17:])}
                    strings = split_title(new['title'])
                    if strings[0] != new['title1'] or strings[1] != new['title2']:
                        colon.append('%s %s %s: %s' % (new['date'], new['title'], new['title1'], new['title2']))
            for key in sorted(count):
                print('%6d %-24s %s' % (count[key]['count'], key, count[key]['string']))
            for value in colon:
                print(value)

if __name__ == '__main__':
    # spiegel = Spiegel(2008, 8)
    # print(spiegel.getContents())
    # news = News(2001, 9, 10)
    # output(news.get_news())
    '''
    x = []
    for d in range(10, 30):
        print('2/%d' % d)
        news = get_news(2008, 2, d)
        for new in news:
            strings = new['url'].split('/')
            string = format_section(strings[3])
            if len(strings) == 6:
                string += '/' + format_subsection(strings[4])
            if not string in x:
                x.append(string)
        print(x)
    '''
    # archive_issues()
    archive_news()



# vi:si:et:sw=4:sts=4:ts=4
# encoding: utf-8
from __future__ import print_function
import re
from urllib.parse import urlencode

from ox.cache import read_url
from ox.html import decode_html, strip_tags
from ox.text import find_re
from ox.text import find_string


# to sniff itunes traffic, use something like
# sudo tcpdump -i en1 -Avs 8192 host appleglobal.112.2o7.net

# http://ax.phobos.apple.com.edgesuite.net/WebObjects/MZSearch.woa/wa/advancedSearch?media=music&songTerm=&genreIndex=1&flavor=0&mediaType=2&composerTerm=&allArtistNames=Arcadia&ringtone=0&searchButton=submit
# http://ax.phobos.apple.com.edgesuite.net/WebObjects/MZSearch.woa/wa/advancedSearch?media=movie&movieTerm=The%20Matrix&descriptionTerm=&ratingIndex=1&mediaType=3&directorProducerName=Andy%20Wachowski&flavor=0&releaseYearTerm=1999&closedCaption=0&actorTerm=&searchButton=submit

ITUNES_HEADERS = {
    'X-Apple-Tz': '0',
    'X-Apple-Storefront': '143441-1',
    'User-Agent': 'iTunes/7.6.2 (Macintosh; U; Intel Mac OS X 10.5.2)',
    'Accept-Language': 'en-us, en;q=0.50',
    'Accept-Encoding': 'gzip',
    'Connection': 'close',
}

def compose_url(request, parameters):
    if request == 'advancedSearch':
        url = 'http://ax.phobos.apple.com.edgesuite.net/WebObjects/MZSearch.woa/wa/advancedSearch?'
        if parameters['media'] == 'music':
            url += urlencode({
              'albumTerm': parameters['title'],
              'allArtistNames': parameters['artist'],
              'composerTerm': '',
              'flavor': 0,
              'genreIndex': 1,
              'media': 'music',
              'mediaType': 2,
              'ringtone': 0,
              'searchButton': 'submit',
              'songTerm': ''
            })
        elif parameters['media'] == 'movie':
            url += urlencode({
              'actorTerm': '',
              'closedCaption': 0,
              'descriptionTerm': '',
              'directorProducerName': parameters['director'],
              'flavor': 0,
              'media': 'movie',
              'mediaType': 3,
              'movieTerm': parameters['title'],
              'ratingIndex': 1,
              'releaseYearTerm': '',
              'searchButton': 'submit'
            })
    elif request == 'viewAlbum':
        url = 'http://phobos.apple.com/WebObjects/MZStore.woa/wa/viewAlbum?id=%s' % parameters['id']
    elif request == 'viewMovie':
        url = 'http://phobos.apple.com/WebObjects/MZStore.woa/wa/viewMovie?id=%s&prvw=1' % parameters['id']
    return url

def parse_xml_dict(xml):
    values = {}
    strings = xml.split('<key>')
    for string in strings:
        if string.find('</key>') != -1:
            key = find_re(string, '(.*?)</key>')
            type = find_re(string, '</key><(.*?)>')
            if type == 'true/':
                value = True
            else:
                value = find_re(string, '<%s>(.*?)</%s>' % (type, type))
                if type == 'integer':
                  value = int(value)
                elif type == 'string':
                  value = decode_html(value)
            values[key] = value
    return values

def parse_cast(xml, title):
    list = []
    try:
        strings = find_re(xml, '<SetFontStyle normalStyle="textColor">%s(.*?)</VBoxView>' % title[:-1].upper()).split('</GotoURL>')
        strings.pop()
        for string in strings:
            list.append(find_re(string, '<SetFontStyle normalStyle="textColor">(.*?)</SetFontStyle>'))
        return list
    except:
        return list

def parse_movies(xml, title):
    list = []
    try:
        strings = find_re(xml, '<SetFontStyle normalStyle="outlineTitleFontStyle"><b>%s(.*?)</Test>' % title[:-1].upper()).split('</GotoURL>')
        strings.pop()
        for string in strings:
            list.append({
              'id': find_re(string, 'viewMovie\?id=(.*?)&'),
              'title': find_re(string, '<SetFontStyle normalStyle="outlineTextFontStyle"><b>(.*?)</b></SetFontStyle>')
            })
        return list
    except:
        return list

class ItunesAlbum:
    def __init__(self, id = '', title = '', artist = ''):
        self.id = id
        self.title = title
        self.artist = artist
        if not id:
            self.id = self.get_id()

    def get_id(self):
        url = compose_url('advancedSearch', {'media': 'music', 'title': self.title, 'artist': self.artist})
        xml = read_url(url, headers = ITUNES_HEADERS)
        id = find_re(xml, 'viewAlbum\?id=(.*?)&')
        return id

    def get_data(self):
        data = {'id': self.id}
        url = compose_url('viewAlbum', {'id': self.id})
        xml = read_url(url, None, ITUNES_HEADERS)
        data['albumName'] = find_re(xml, '<B>(.*?)</B>')
        data['artistName'] = find_re(xml, '<b>(.*?)</b>')
        data['coverUrl'] = find_re(xml, 'reflection="." url="(.*?)"')
        data['genre'] = find_re(xml, 'Genre:(.*?)<')
        data['releaseDate'] = find_re(xml, 'Released(.*?)<')
        data['review'] = strip_tags(find_re(xml, 'REVIEW</b>.*?<SetFontStyle normalStyle="textColor">(.*?)</SetFontStyle>'))
        data['tracks'] = []
        strings = find_re(xml, '<key>items</key>.*?<dict>(.*?)$').split('<dict>')
        for string in strings:
          data['tracks'].append(parse_xml_dict(string))
        data['type'] = find_re(xml, '<key>listType</key><string>(.*?)<')
        return data

class ItunesMovie:
    def __init__(self, id = '', title = '', director = ''):
        self.id = id
        self.title = title
        self.director = director
        if not id:
            self.id = self.get_id()

    def get_id(self):
        url = compose_url('advancedSearch', {'media': 'movie', 'title': self.title, 'director': self.director})
        xml = read_url(url, headers = ITUNES_HEADERS)
        id = find_re(xml, 'viewMovie\?id=(.*?)&')
        return id

    def get_data(self):
        data = {'id': self.id}
        url = compose_url('viewMovie', {'id': self.id})
        xml = read_url(url, None, ITUNES_HEADERS)
        f = open('/Users/rolux/Desktop/iTunesData.xml', 'w')
        f.write(xml)
        f.close()
        data['actors'] = parse_cast(xml, 'actors')
        string = find_re(xml, 'Average Rating:(.*?)</HBoxView>')
        data['averageRating'] = string.count('rating_star_000033.png') + string.count('&#189;') * 0.5
        data['directors'] = parse_cast(xml, 'directors')
        data['format'] = find_re(xml, 'Format:(.*?)<')
        data['genre'] = decode_html(find_re(xml, 'Genre:(.*?)<'))
        data['plotSummary'] = decode_html(find_re(xml, 'PLOT SUMMARY</b>.*?<SetFontStyle normalStyle="textColor">(.*?)</SetFontStyle>'))
        data['posterUrl'] = find_re(xml, 'reflection="." url="(.*?)"')
        data['producers'] = parse_cast(xml, 'producers')
        data['rated'] = find_re(xml, 'Rated(.*?)<')
        data['relatedMovies'] = parse_movies(xml, 'related movies')
        data['releaseDate'] = find_re(xml, 'Released(.*?)<')
        data['runTime'] = find_re(xml, 'Run Time:(.*?)<')
        data['screenwriters'] = parse_cast(xml, 'screenwriters')
        data['soundtrackId'] = find_re(xml, 'viewAlbum\?id=(.*?)&')
        data['trailerUrl'] = find_re(xml, 'autoplay="." url="(.*?)"')
        return data

if __name__ == '__main__':
    from ox.utils import json
    data = ItunesAlbum(title = 'So Red the Rose', artist = 'Arcadia').get_data()
    print(json.dumps(data, sort_keys = True, indent = 4))
    data = ItunesMovie(title = 'The Matrix', director = 'Wachowski').get_data()
    print(json.dumps(data, sort_keys = True, indent = 4))
    for v in data['relatedMovies']:
        data = ItunesMovie(id = v['id']).get_data()
        print(json.dumps(data, sort_keys = True, indent = 4))
    data = ItunesMovie(id='272960052').get_data()
    print(json.dumps(data, sort_keys = True, indent = 4))




# -*- coding: utf-8 -*-
# vi:si:et:sw=4:sts=4:ts=4
import re
from urllib.parse import unquote
from ox.cache import read_url


def get_video_url(url):
    '''
    >>> get_video_url('http://www.dailymotion.com/relevance/search/priere%2Bpour%2Brefuznik/video/x3opar_priere-pour-refuznik-1-jeanluc-goda_shortfilms').split('?auth')[0]
    'http://www.dailymotion.com/cdn/FLV-320x240/video/x3opar_priere-pour-refuznik-1-jean-luc-god_shortfilms.flv'

    >>> get_video_url('http://www.dailymotion.com/relevance/search/priere%2Bpour%2Brefuznik/video/x3ou94_priere-pour-refuznik-2-jeanluc-goda_shortfilms').split('?auth')[0]
    'http://www.dailymotion.com/cdn/FLV-320x240/video/x3ou94_priere-pour-refuznik-2-jean-luc-god_shortfilms.flv'
    '''
    data = read_url(url)
    video = re.compile('''video", "(.*?)"''').findall(data)
    for v in video:
        v = unquote(v).split('@@')[0]
        return v
    return ''



# -*- coding: utf-8 -*-
# vi:si:et:sw=4:sts=4:ts=4
import re
import time

from ox import strip_tags, find_re
from ox.cache import read_url


def get_episode_data(url):
    '''
      prases informatin on tvcom episode pages
      returns dict with title, show, description, score
      example:
        get_episode_data('http://www.tv.com/lost/do-no-harm/episode/399310/summary.html')
    '''
    data = read_url(url, unicode=True)
    r = {}
    r['description'] = strip_tags(find_re(data, 'div id="main-col">.*?<div>(.*?)</div').split('\r')[0])
    r['show'] = find_re(data, '<h1>(.*?)</h1>')
    r['title'] =  find_re(data, '<title>.*?: (.*?) - TV.com  </title>')
    #episode score
    r['episode score'] = find_re(data, '<span class="f-28 f-bold mt-10 mb-10 f-FF9 db lh-18">(.*?)</span>')

    match = re.compile('Episode Number: (\d*?) &nbsp;&nbsp; Season Num: (\d*?) &nbsp;&nbsp; First Aired: (.*?) &nbsp').findall(data) 
    if match:
        r['season'] = int(match[0][1])
        r['episode'] = int(match[0][0])
        #'Wednesday September 29, 2004' -> 2004-09-29 
        r['air date'] = time.strftime('%Y-%m-%d', time.strptime(match[0][2], '%A %B %d, %Y'))
    return r




# -*- coding: utf-8 -*-
# vi:si:et:sw=4:sts=4:ts=4
import codecs

import ox
from . import srt

def _webvtt_timecode(t):
    return ox.format_duration(t * 1000, years=False)


def encode(data, webvtt=False):
    """Encodes subtitles into WebVTT format

    data: list of dicts with 'in', 'out': float and 'value': unicode

    Returns: a UTF-8-encoded bytestring

    >>> encode([{'in': 1.25, 'out': 60 * 60 + 1, 'value': u'touch\\u00E9'}])
    '\\xef\\xbb\\xbfWEBVTT\\r\\n\\r\\n1\\r\\n00:00:01.250 --> 01:00:01.000\\r\\ntouch\\xc3\\xa9\\r\\n\\r\\n'
    """
    srt = u'WEBVTT\r\n\r\n'

    for i, s in enumerate(data, 1):
        srt += '%d\r\n%s --> %s\r\n%s\r\n\r\n' % (
            i,
            _webvtt_timecode(s['in']),
            _webvtt_timecode(s['out']),
            s['value'].replace('\n', '\r\n').strip()
        )

    return codecs.BOM_UTF8 + srt.encode('utf-8')

def load(filename, offset=0):
    '''Parses vtt file

    filename: path to an vtt file
    offset (float, seconds): shift all in/out points by offset

    Returns list with dicts that have in, out, value and id
    '''
    return srt.load(filename, offset)



from ox.cache import read_url
import re
import lxml.html

def get_data(id):
    info = {}
    base = 'http://www.abebooks.com'
    url = '%s/servlet/SearchResults?isbn=%s&sts=t' % (base, id)
    data = read_url(url)
    urls = re.compile('href="(/servlet/BookDetailsPL[^"]+)"').findall(data)
    if urls:
        details = '%s%s' % (base, urls[0])
        data = read_url(details)
        doc = lxml.html.document_fromstring(data)
        for e in doc.xpath("//*[contains(@id, 'biblio')]"):
            key = e.attrib['id'].replace('biblio-', '')
            value = e.text_content()
            if value and key not in ('bookcondition', 'binding'):
                info[key] = value
    return info



# -*- coding: utf-8 -*-
# vi:si:et:sw=4:sts=4:ts=4
import re
import string
import subprocess
import ox
import os

def get_ids():
    result = []
    for i in string.ascii_uppercase:
        url = "http://www.filmsdivision.org/search.php?title=%s" % i
        data = ox.cache.read_url(url)
        links = re.compile('view_video.php\?movId=(.*?)[\'"]', re.DOTALL).findall(data)
        result += links
    return list(set(result))

def get_data(id):
    result = {}
    url = "http://www.filmsdivision.org/view_video.php?movId=%s" % id
    data = ox.cache.read_url(url)
    result['title'] = re.compile('<td.*?class="vdoheadtxt".*?>(.*?)</td>').findall(data)[0]
    result['year'] = re.compile('Release: (\d{4})').findall(data)[0]
    result['duration'] = int(re.compile('Duration: (\d+)mins').findall(data)[0]) * 60
    result['producer'] = re.compile('Producer: (.*?)\t').findall(data)[0].strip()
    if 'Director:' in data:
        result['director'] = re.compile('Director: (.*?)\t').findall(data)[0].strip()
    else:
        result['director'] = "Unknown Director"
    result['url'] = re.compile('value="(.*?.wmv)"').findall(data)[0]
    return result

def download_video(url, filename):
    dirname = os.path.dirname(filename)
    if not os.path.exists(dirname):
        os.makedirs(dirname)
    p = subprocess.Popen(['gst-launch', 'mmssrc', 'location=%s'%url, '!', 'filesink', 'locaiton='%filename])
    p.wait()
    return p.returncode == 0



# -*- coding: UTF-8 -*-
# vi:si:et:sw=4:sts=4:ts=4

import re
from lxml.html import document_fromstring

from ox.cache import read_url
from ox import find_re, strip_tags
from ox.web.imdb import ImdbCombined


def get_data(id, timeout=-1):
    '''
    >>> get_data('the-matrix')['poster']
    'http://content7.flixster.com/movie/16/90/52/1690525_gal.jpg'

    >>> get_data('0133093')['poster']
    'http://content7.flixster.com/movie/16/90/52/1690525_gal.jpg'

    >>> get_data('2-or-3-things-i-know-about-her')['poster']
    'http://content6.flixster.com/movie/10/95/43/10954392_gal.jpg'

    >>> get_data('0078875')['rottentomatoes_id']
    'http://www.rottentomatoes.com/m/the-tin-drum/'
    '''
    if len(id) == 7:
        try:
            int(id)
            id = get_id(imdb=id)
        except:
            pass
    data = {
        "url": get_url(id),
    }
    html = read_url(data['url'], timeout=timeout, unicode=True)
    doc = document_fromstring(html)

    props = {
        'og:title': 'title',
        'og:image': 'poster',
        'og:url': 'rottentomatoes_id',
    }
    for meta in doc.head.findall('meta'):
        prop = meta.attrib.get('property', None)
        content = meta.attrib.get('content', '')
        if prop in props and content:
            data[props[prop]] = content

    for p in doc.body.find_class('synopsis'):
        data['synopsis'] = p.text.strip()

    if 'poster' in data and data['poster']:
        data['poster'] = data['poster'].replace('_pro.jpg', '_gal.jpg')
    if not 'title' in data:
        return None
    return data

def get_id(url=None, imdb=None):
    '''
    >>> get_id(imdb='0133093')
    'the-matrix'

    #>>> get_id(imdb='0060304')
    #'2-or-3-things-i-know-about-her'
    '''
    if imdb:
        i = ImdbCombined(imdb)
        title = i['title']
        return title.replace(' ', '-').lower().replace("'", '')
    return url.split('/')[-1]

def get_url(id):
    return "http://www.flixster.com/movie/%s"%id




# -*- coding: utf-8 -*-
# vi:si:et:sw=4:sts=4:ts=4
# GPL 2008

_iso639_languages = [
    ("Unknown", "", "", "und"),
    ("Afar", "", "aa", "﻿aar"),
    ("Abkhazian", "", "ab", "abk"),
    ("Afrikaans", "", "af", "afr"),
    ("Akan", "", "ak", "aka"),
    ("Albanian", "", "sq", "sqi"),
    ("Amharic", "", "am", "amh"),
    ("Arabic", "", "ar", "ara"),
    ("Aragonese", "", "an", "arg"),
    ("Armenian", "", "hy", "hye"),
    ("Assamese", "", "as", "asm"),
    ("Avaric", "", "av", "ava"),
    ("Avestan", "", "ae", "ave"),
    ("Aymara", "", "ay", "aym"),
    ("Azerbaijani", "", "az", "aze"),
    ("Bashkir", "", "ba", "bak"),
    ("Bambara", "", "bm", "bam"),
    ("Basque", "", "eu", "eus"),
    ("Belarusian", "", "be", "bel"),
    ("Bengali", "", "bn", "ben"),
    ("Bihari", "", "bh", "bih"),
    ("Bislama", "", "bi", "bis"),
    ("Bosnian", "", "bs", "bos"),
    ("Breton", "", "br", "bre"),
    ("Bulgarian", "", "bg", "bul"),
    ("Burmese", "", "my", "mya"),
    ("Catalan", "", "ca", "cat"),
    ("Chamorro", "", "ch", "cha"),
    ("Chechen", "", "ce", "che"),
    ("Chinese", "", "zh", "zho"),
    ("Church Slavic", "", "cu", "chu"),
    ("Chuvash", "", "cv", "chv"),
    ("Cornish", "", "kw", "cor"),
    ("Corsican", "", "co", "cos"),
    ("Cree", "", "cr", "cre"),
    ("Czech", "", "cs", "ces"),
    ("Danish", "Dansk", "da", "dan"),
    ("Divehi", "", "dv", "div"),
    ("Dutch", "Nederlands", "nl", "nld"),
    ("Dzongkha", "", "dz", "dzo"),
    ("English", "English", "en", "eng"),
    ("Esperanto", "", "eo", "epo"),
    ("Estonian", "", "et", "est"),
    ("Ewe", "", "ee", "ewe"),
    ("Faroese", "", "fo", "fao"),
    ("Fijian", "", "fj", "fij"),
    ("Finnish", "Suomi", "fi", "fin"),
    ("French", "Francais", "fr", "fra"),
    ("Western Frisian", "", "fy", "fry"),
    ("Fulah", "", "ff", "ful"),
    ("Georgian", "", "ka", "kat"),
    ("German", "Deutsch", "de", "deu"),
    ("Gaelic (Scots)", "", "gd", "gla"),
    ("Irish", "", "ga", "gle"),
    ("Galician", "", "gl", "glg"),
    ("Manx", "", "gv", "glv"),
    ("Greek, Modern", "", "el", "ell"),
    ("Guarani", "", "gn", "grn"),
    ("Gujarati", "", "gu", "guj"),
    ("Haitian", "", "ht", "hat"),
    ("Hausa", "", "ha", "hau"),
    ("Hebrew", "", "he", "heb"),
    ("Herero", "", "hz", "her"),
    ("Hindi", "", "hi", "hin"),
    ("Hiri Motu", "", "ho", "hmo"),
    ("Hungarian", "Magyar", "hu", "hun"),
    ("Igbo", "", "ig", "ibo"),
    ("Icelandic", "Islenska", "is", "isl"),
    ("Ido", "", "io", "ido"),
    ("Sichuan Yi", "", "ii", "iii"),
    ("Inuktitut", "", "iu", "iku"),
    ("Interlingue", "", "ie", "ile"),
    ("Interlingua", "", "ia", "ina"),
    ("Indonesian", "", "id", "ind"),
    ("Inupiaq", "", "ik", "ipk"),
    ("Italian", "Italiano", "it", "ita"),
    ("Javanese", "", "jv", "jav"),
    ("Japanese", "", "ja", "jpn"),
    ("Kalaallisut (Greenlandic)", "", "kl", "kal"),
    ("Kannada", "", "kn", "kan"),
    ("Kashmiri", "", "ks", "kas"),
    ("Kanuri", "", "kr", "kau"),
    ("Kazakh", "", "kk", "kaz"),
    ("Central Khmer", "", "km", "khm"),
    ("Kikuyu", "", "ki", "kik"),
    ("Kinyarwanda", "", "rw", "kin"),
    ("Kirghiz", "", "ky", "kir"),
    ("Komi", "", "kv", "kom"),
    ("Kongo", "", "kg", "kon"),
    ("Korean", "", "ko", "kor"),
    ("Kuanyama", "", "kj", "kua"),
    ("Kurdish", "", "ku", "kur"),
    ("Lao", "", "lo", "lao"),
    ("Latin", "", "la", "lat"),
    ("Latvian", "", "lv", "lav"),
    ("Limburgan", "", "li", "lim"),
    ("Lingala", "", "ln", "lin"),
    ("Lithuanian", "", "lt", "lit"),
    ("Luxembourgish", "", "lb", "ltz"),
    ("Luba-Katanga", "", "lu", "lub"),
    ("Ganda", "", "lg", "lug"),
    ("Macedonian", "", "mk", "mkd"),
    ("Marshallese", "", "mh", "mah"),
    ("Malayalam", "", "ml", "mal"),
    ("Maori", "", "mi", "mri"),
    ("Marathi", "", "mr", "mar"),
    ("Malay", "", "ms", "msa"),
    ("Malagasy", "", "mg", "mlg"),
    ("Maltese", "", "mt", "mlt"),
    ("Moldavian", "", "mo", "mol"),
    ("Mongolian", "", "mn", "mon"),
    ("Nauru", "", "na", "nau"),
    ("Navajo", "", "nv", "nav"),
    ("Ndebele, South", "", "nr", "nbl"),
    ("Ndebele, North", "", "nd", "nde"),
    ("Ndonga", "", "ng", "ndo"),
    ("Nepali", "", "ne", "nep"),
    ("Norwegian Nynorsk", "", "nn", "nno"),
    ("Norwegian Bokmål", "", "nb", "nob"),
    ("Norwegian", "Norsk", "no", "nor"),
    ("Chichewa; Nyanja", "", "ny", "nya"),
    ("Occitan (post 1500); Provençal", "", "oc", "oci"),
    ("Ojibwa", "", "oj", "oji"),
    ("Oriya", "", "or", "ori"),
    ("Oromo", "", "om", "orm"),
    ("Ossetian; Ossetic", "", "os", "oss"),
    ("Panjabi", "", "pa", "pan"),
    ("Persian", "", "fa", "fas"),
    ("Pali", "", "pi", "pli"),
    ("Polish", "", "pl", "pol"),
    ("Portuguese", "Portugues", "pt", "por"),
    ("Pushto", "", "ps", "pus"),
    ("Quechua", "", "qu", "que"),
    ("Romansh", "", "rm", "roh"),
    ("Romanian", "", "ro", "ron"),
    ("Rundi", "", "rn", "run"),
    ("Russian", "", "ru", "rus"),
    ("Sango", "", "sg", "sag"),
    ("Sanskrit", "", "sa", "san"),
    ("Serbian", "", "sr", "srp"),
    ("Croatian", "Hrvatski", "hr", "hrv"),
    ("Sinhala", "", "si", "sin"),
    ("Slovak", "", "sk", "slk"),
    ("Slovenian", "", "sl", "slv"),
    ("Northern Sami", "", "se", "sme"),
    ("Samoan", "", "sm", "smo"),
    ("Shona", "", "sn", "sna"),
    ("Sindhi", "", "sd", "snd"),
    ("Somali", "", "so", "som"),
    ("Sotho, Southern", "", "st", "sot"),
    ("Spanish", "Espanol", "es", "spa"),
    ("Sardinian", "", "sc", "srd"),
    ("Swati", "", "ss", "ssw"),
    ("Sundanese", "", "su", "sun"),
    ("Swahili", "", "sw", "swa"),
    ("Swedish", "Svenska", "sv", "swe"),
    ("Tahitian", "", "ty", "tah"),
    ("Tamil", "", "ta", "tam"),
    ("Tatar", "", "tt", "tat"),
    ("Telugu", "", "te", "tel"),
    ("Tajik", "", "tg", "tgk"),
    ("Tagalog", "", "tl", "tgl"),
    ("Thai", "", "th", "tha"),
    ("Tibetan", "", "bo", "bod"),
    ("Tigrinya", "", "ti", "tir"),
    ("Tonga (Tonga Islands)", "", "to", "ton"),
    ("Tswana", "", "tn", "tsn"),
    ("Tsonga", "", "ts", "tso"),
    ("Turkmen", "", "tk", "tuk"),
    ("Turkish", "", "tr", "tur"),
    ("Twi", "", "tw", "twi"),
    ("Uighur", "", "ug", "uig"),
    ("Ukrainian", "", "uk", "ukr"),
    ("Urdu", "", "ur", "urd"),
    ("Uzbek", "", "uz", "uzb"),
    ("Venda", "", "ve", "ven"),
    ("Vietnamese", "", "vi", "vie"),
    ("Volapük", "", "vo", "vol"),
    ("Welsh", "", "cy", "cym"),
    ("Walloon", "", "wa", "wln"),
    ("Wolof", "", "wo", "wol"),
    ("Xhosa", "", "xh", "xho"),
    ("Yiddish", "", "yi", "yid"),
    ("Yoruba", "", "yo", "yor"),
    ("Zhuang", "", "za", "zha"),
    ("Zulu", "", "zu", "zul"),
]

def codeToLang(code):
    if code:
        code = code.lower()
        if len(code) == 2:
            for l in _iso639_languages:
                if l[2] == code:
                    return l[0]
        elif len(code) == 3:
            for l in _iso639_languages:
                if l[3] == code:
                    return l[0]
    return None

def langTo3Code(lang):
    if lang:
        lang = langEnglishName(lang)
        if lang:
            lang = lang.lower()
        for l in _iso639_languages:
            if l[0].lower() == lang:
                return l[3]
    return None

def langTo2Code(lang):
    if lang:
        lang = langEnglishName(lang)
        if lang:
            lang = lang.lower()
        for l in _iso639_languages:
            if l[0].lower() == lang:
                return l[2]
    return None

def langCode2To3(code):
    return langTo3Code(codeToLang(code))

def langCode3To2(code):
    return langTo2Code(codeToLang(code))

def langEnglishName(lang):
    lang = lang.lower()
    for l in _iso639_languages:
        if l[1].lower() == lang or l[0].lower() == lang:
            return l[0]
    return None

def languages2Letter():
    languages = []
    for l in _iso639_languages:
        if l[2]:
            languages.append(l[2])
    return languages




# -*- coding: utf-8 -*-
# vi:si:et:sw=4:sts=4:ts=4
import re

from urllib.parse import quote
from lxml.html import document_fromstring

from ox.cache import read_url
from ox import find_re, strip_tags

def get_url(id=None, imdb=None):
    if imdb:
        url = "http://www.imdb.com/title/tt%s/criticreviews" % imdb
        data = read_url(url)
        metacritic_url = find_re(data, '"(http://www.metacritic.com/movie/.*?)"')
        return metacritic_url or None
    return 'http://www.metacritic.com/movie/%s' % id

def get_id(url):
    return url.split('/')[-1]

def get_show_url(title):
    title = quote(title)
    url = "http://www.metacritic.com/search/process?ty=6&ts=%s&tfs=tvshow_title&x=0&y=0&sb=0&release_date_s=&release_date_e=&metascore_s=&metascore_e=" % title
    data = read_url(url)
    return find_re(data, '(http://www.metacritic.com/tv/shows/.*?)\?')

def get_data(url):
    data = read_url(url, unicode=True)
    doc = document_fromstring(data)
    score = [s for s in doc.xpath('//span[@class="score_value"]')
             if s.attrib.get('property') == 'v:average']
    if score:
        score = int(score[0].text)
    else:
        score = -1
    authors = [
        a.text
        for a in doc.xpath('//div[@class="review_content"]//div[@class="author"]//a')
    ]
    sources = [
        d.text
        for d in doc.xpath('//div[@class="review_content"]//div[@class="source"]/a')
    ]
    reviews = [
        d.text
        for d in doc.xpath('//div[@class="review_content"]//div[@class="review_body"]')
    ]
    scores = [
        int(d.text.strip())
        for d in doc.xpath('//div[@class="review_content"]//div[contains(@class, "critscore")]')
    ]
    urls = [
        a.attrib['href']
        for a in doc.xpath('//div[@class="review_content"]//a[contains(@class, "external")]')
    ]

    metacritics = []
    for i in range(len(authors)):
        metacritics.append({
            'critic': authors[i],
            'url': urls[i],
            'source': sources[i],
            'quote': strip_tags(reviews[i]).strip(),
            'score': scores[i],
        })

    return {
        'critics': metacritics,
        'id': get_id(url),
        'score': score,
        'url': url,
    }




# -*- coding: utf-8 -*-
# vi:si:et:sw=4:sts=4:ts=4
import re

from ox.cache import read_url
from ox import find_re, strip_tags
from ox.iso import langCode2To3, langTo3Code

def find_subtitles(imdb, parts = 1, language = "eng"):
    import feedparser
    if len(language) == 2:
        language = langCode2To3(language)
    elif len(language) != 3:
        language = langTo3Code(language)
    url = "http://www.opensubtitles.org/en/search/"
    if language:
        url += "sublanguageid-%s/" % language
    url += "subsumcd-%s/subformat-srt/imdbid-%s/rss_2_00" % (parts, imdb)
    data = read_url(url)
    if "title>opensubtitles.com - search results</title" in data:
        fd = feedparser.parse(data)
        opensubtitleId = None
        if fd.entries:
            link = fd.entries[0]['links'][0]['href']
            opensubtitleId = re.compile('subtitles/(.*?)/').findall(link)
            if opensubtitleId:
                opensubtitleId = opensubtitleId[0]
    else:
        opensubtitleId = find_re(data, '/en/subtitles/(.*?)/')
    return opensubtitleId

def download_subtitle(opensubtitle_id):
    srts = {}
    data = read_url('http://www.opensubtitles.org/en/subtitles/%s' % opensubtitle_id)
    reg_exp = 'href="(/en/download/file/.*?)">(.*?)</a>'
    for f in re.compile(reg_exp, re.DOTALL).findall(data):
        name = strip_tags(f[1]).split('\n')[0]
        url = "http://www.opensubtitles.com%s" % f[0]
        srts[name] = read_url(url, unicode=True)
    return srts




# -*- coding: utf-8 -*-
# vi:si:et:sw=4:sts=4:ts=4
# GPL 2011
try:
    from . import __version
    __version__ = __version.VERSION
except:
    __version__ = '3.0.10'

from . import cache
from . import js
from . import jsonc
from . import net
from . import srt
from . import utils
from . import vtt

from .api import *
from .file import *
from .format import *
from .geo import *
from .html import *
#image depends on PIL, not easy enough to instal on osx
try:
    from .image import *
except:
    pass
from .location import *
from .movie import *
from .normalize import *
from .oembed import *
from .text import *
#currently broken in python3
try:
    from .torrent import *
except:
    pass
from .fixunicode import *



# -*- coding: utf-8 -*-
# vi:si:et:sw=4:sts=4:ts=4
import re
import urllib

import ox
from ox import strip_tags, decode_html

DEFAULT_MAX_RESULTS = 10
DEFAULT_TIMEOUT = 24*60*60

def read_url(url, data=None, headers=ox.net.DEFAULT_HEADERS, timeout=DEFAULT_TIMEOUT):
    return ox.cache.read_url(url, data, headers, timeout, unicode=True)

def quote_plus(s):
    if not isinstance(s, bytes):
        s = s.encode('utf-8')
    return urllib.parse.quote_plus(s)


def infobox(query, timeout=DEFAULT_TIMEOUT):
    import lxml.html
    data = read_url(url, timeout=timeout)
    doc = lxml.html.document_fromstring(data)
    k = 'kp-wholepage'
    wholepage = doc.cssselect('.' + k)
    infobox = {}
    if wholepage:
        page = wholepage[0]
        for a in page.cssselect('a'):
            if a.attrib.get('href', '').startswith('http'):
                domain = '.'.join(a.attrib['href'].split('/')[2].split('.')[-2:])
                infobox[domain] = a.attrib['href']
        for e in page.cssselect('*[data-attrid]'):
            key = e.attrib['data-attrid']
            value = e.text_content()
            if value and key not in (
                'kc:/film/film:media_actions_wholepage',
                'action:watch_film'
            ):
                infobox[key] = value
    return infobox


def find(query, max_results=DEFAULT_MAX_RESULTS, timeout=DEFAULT_TIMEOUT):
    """
    Return max_results tuples with title, url, description 

    >>> str(find("The Matrix site:imdb.com", 1)[0][0])
    'The Matrix (1999) - IMDb'

    >>> str(find("The Matrix site:imdb.com", 1)[0][1])
    'http://www.imdb.com/title/tt0133093/'
    """
    results = []
    offset = 0
    while len(results) < max_results:
        url = 'http://google.com/search?q=%s' % quote_plus(query)
        if offset:
            url += '&start=%d' % offset
        data = read_url(url, timeout=timeout)
        data = re.sub('<span class="f">(.*?)</span>', '\\1', data)
        for a in re.compile('<a href="(htt\S+?)".*?>(.*?)</a>.*?<span class="st">(.*?)<\/span>').findall(data):
            results.append((strip_tags(decode_html(a[1])), a[0], strip_tags(decode_html(a[2]))))
            if len(results) >= max_results:
                break
        offset += 10
    return results




# -*- coding: utf-8 -*-
# vi:si:et:sw=4:sts=4:ts=4
from .. import cache
from ..utils import json

def get_id(url):
    return url.split("/")[-1]

def get_url(id):
    return "http://www.archive.org/details/%s" % id

def get_data(id):
    data = {}
    url = get_url(id)
    details = cache.read_url('%s?output=json' % url)
    details = json.loads(details)
    for key in ('title', 'description', 'runtime'):
        if key in details['metadata']:
            data[key] = details['metadata'][key]
            if isinstance(data[key], list):
                data[key] = data[key][0]
            if isinstance(data[key], str):
                data[key] = data[key].strip()
                if data[key][0] == '[' and data[key][-1] == ']':
                    data[key] = data[key][1:-1] 
    data['url'] = url
    data['image'] = 'http://archive.org/download/%s/format=thumbnail' % id
    data['ogg'] = 'http://archive.org/download/%s/format=Ogg+video' % id
    data['mp4'] = 'http://archive.org/download/%s/format=512Kb+MPEG4' % id
    return data




# -*- coding: utf-8 -*-
# vi:si:et:sw=4:sts=4:ts=4
from __future__ import print_function
import re

from ox import strip_tags, find_re
from ox.cache import read_url


def get_id(url):
    return url.split("/")[-1]

def get_data(id):
    '''
    >>> get_data('129689')['cast'][1][1]
    'Marianne'
    >>> get_data('129689')['credits'][0][0]
    'Jean-Luc Godard'
    >>> get_data('129689')['posters'][0]
    'http://image.allmusic.com/00/adg/cov200/dru800/u812/u81260bbffr.jpg'
    >>> get_data('129689')['rating']
    '4.5'
    '''
    if id.startswith('http'):
        id = get_id(id)
    data = {
        "url": get_url(id)
    }
    html = read_url(data["url"], unicode=True)
    data['aka'] = parse_list(html, 'AKA')
    data['category'] = find_re(html, '<dt>category</dt>.*?<dd>(.*?)</dd>')
    data['countries'] = parse_list(html, 'countries')
    data['director'] = parse_entry(html, 'directed by')
    data['genres'] = parse_list(html, 'genres')
    data['keywords'] = parse_list(html, 'keywords')
    data['posters'] = [find_re(html, '<img src="(http://cps-.*?)"')]
    data['produced'] = parse_list(html, 'produced by')
    data['rating'] = find_re(html, 'Stars" title="(.*?) Stars"')
    data['released'] = parse_entry(html, 'released by')
    data['releasedate'] = parse_list(html, 'release date')
    data['runtime'] = parse_entry(html, 'run time').replace('min.', '').strip()
    data['set'] = parse_entry(html, 'set in')
    data['synopsis'] = strip_tags(find_re(html, '<div class="toggle-text" itemprop="description">(.*?)</div>')).strip()
    data['themes'] = parse_list(html, 'themes')
    data['types'] = parse_list(html, 'types')
    data['year'] = find_re(html, '<span class="year">.*?(\d+)')
    #data['stills'] = [re.sub('_derived.*?/', '', i) for i in re.compile('<a href="#" title="movie still".*?<img src="(.*?)"', re.DOTALL).findall(html)]
    data['stills'] = re.compile('<a href="#" title="movie still".*?<img src="(.*?)"', re.DOTALL).findall(html)
    #html = read_url("http://allmovie.com/work/%s/cast" % id, unicode=True)
    #data['cast'] = parse_table(html)
    #html = read_url("http://allmovie.com/work/%s/credits" % id, unicode=True)
    #data['credits'] = parse_table(html)
    html = read_url("http://allmovie.com/work/%s/review" % id, unicode=True)
    data['review'] = strip_tags(find_re(html, '<div class="toggle-text" itemprop="description">(.*?)</div>')).strip()
    return data

def get_url(id):
    return "http://allmovie.com/work/%s" % id

def parse_entry(html, title):
    html = find_re(html, '<dt>%s</dt>.*?<dd>(.*?)</dd>' % title)
    return strip_tags(html).strip()

def parse_list(html, title):
    html = find_re(html, '<dt>%s</dt>.*?<dd>(.*?)</dd>' % title.lower())
    r = map(strip_tags, re.compile('<li>(.*?)</li>', re.DOTALL).findall(html))
    if not r and html:
        r = [strip_tags(html)]
    return r

def parse_table(html):
    return [
        [
            strip_tags(r).strip().replace('&nbsp;', '')
            for r in x.split('<td width="305">-')
        ]
        for x in find_re(html, '<div id="results-table">(.*?)</table>').split('</tr>')[:-1]
    ]

def parse_text(html, title):
    return strip_tags(find_re(html, '%s</td>.*?<td colspan="2"><p>(.*?)</td>' % title)).strip()

if __name__ == '__main__':
    print(get_data('129689'))
    # print(get_data('177524'))




# -*- coding: utf-8 -*-
# vi:si:et:sw=4:sts=4:ts=4
# GPL 2008
from __future__ import division, print_function
from distutils.spawn import find_executable
from glob import glob
import hashlib
import os
import re
import shutil
import sqlite3
import struct
import subprocess

from .utils import json

__all__ = ['sha1sum', 'oshash', 'avinfo', 'makedirs', 'iexists']

EXTENSIONS = {
    'audio': [
        'aac', 'aif', 'aiff', 'amr',
        'flac', 'm4a', 'mp3', 'oga', 'ogg', 'wav', 'wma', 'opus',
        'ra',  # Real Audio
    ],
    'image': [
        'bmp', 'gif', 'jpeg', 'jpg', 'png', 'svg', 'webp'
    ],
    'subtitle': [
        'idx', 'srt', 'sub', 'vtt'
    ],
    'video': [
        '3gp',
        'avi', 'divx', 'dv', 'flv', 'm2t', 'm2ts', 'm2v', 'm4v', 'mkv', 'mov', 'mp4',
        'mpeg', 'mpg', 'mts', 'ogm', 'ogv', 'vob', 'webm', 'wmv', 'asf',
        'mod', 'tod',  # http://en.wikipedia.org/wiki/MOD_and_TOD
        'mxf', 'ts',
        'dat',  # VOD files
        'rm', 'rmvb',  # Real Media
    ],
}

def cmd(program):
    local = os.path.expanduser('~/.ox/bin/%s' % program)
    if os.path.exists(local):
        program = local
    return program

def _get_file_cache():
    import ox.cache
    path = ox.cache.cache_path()
    if path.startswith('fs:'):
        path = path[3:]
    return os.path.join(path, 'files.sqlite')

def cache(filename, type='oshash', update=False):
    conn = sqlite3.connect(_get_file_cache(), timeout=10)
    conn.row_factory = sqlite3.Row

    if not cache.init:
        c = conn.cursor()
        c.execute('CREATE TABLE IF NOT EXISTS cache (path varchar(1024) unique, oshash varchar(16), sha1 varchar(42), size int, mtime int, info text)')
        c.execute('CREATE INDEX IF NOT EXISTS cache_oshash ON cache (oshash)')
        c.execute('CREATE INDEX IF NOT EXISTS cache_sha1 ON cache (sha1)')
        conn.commit()
        cache.init = True
    c = conn.cursor()
    c.execute('SELECT oshash, sha1, info, size, mtime FROM cache WHERE path = ?', (filename, ))
    stat = os.stat(filename)
    row = None
    h = None
    sha1 = None
    info = ''
    for row in c:
        if stat.st_size == row['size'] and int(stat.st_mtime) == int(row['mtime']):
            if not update:
                value = row[type]
                if value:
                    if type == 'info':
                        value = json.loads(value)
                    return value
            h = row['oshash']
            sha1 = row['sha1']
            info = row['info']
    if type == 'oshash':
        value = h = oshash(filename, cached=False)
    elif type == 'sha1':
        value = sha1 = sha1sum(filename, cached=False)
    elif type == 'info':
        value = avinfo(filename, cached=False)
        info = json.dumps(value)
    t = (filename, h, sha1, stat.st_size, int(stat.st_mtime), info)
    with conn:
        sql = u'INSERT OR REPLACE INTO cache values (?, ?, ?, ?, ?, ?)'
        c.execute(sql, t)
    return value
cache.init = None

def cleanup_cache():
    conn = sqlite3.connect(_get_file_cache(), timeout=10)
    conn.row_factory = sqlite3.Row
    c = conn.cursor()
    c.execute('SELECT path FROM cache')
    paths = [r[0] for r in c]
    for path in paths:
        if not os.path.exists(path):
            c.execute('DELETE FROM cache WHERE path = ?', (path, ))
    conn.commit()
    c.execute('VACUUM')
    conn.commit()

def sha1sum(filename, cached=False):
    if cached:
        return cache(filename, 'sha1')
    sha1 = hashlib.sha1()
    with open(filename, 'rb') as f:
        for chunk in iter(lambda: f.read(128*sha1.block_size), b''):
            sha1.update(chunk)
    return sha1.hexdigest()

'''
    os hash - http://trac.opensubtitles.org/projects/opensubtitles/wiki/HashSourceCodes
    plus modification for files < 64k, buffer is filled with file data and padded with 0
'''
def oshash(filename, cached=True):
    if cached:
        return cache(filename, 'oshash')
    try:
        longlongformat = 'q'  # long long
        bytesize = struct.calcsize(longlongformat)

        f = open(filename, "rb")

        filesize = os.path.getsize(filename)
        hash = filesize
        if filesize < 65536:
            for x in range(int(filesize/bytesize)):
                buffer = f.read(bytesize)
                (l_value,) = struct.unpack(longlongformat, buffer)
                hash += l_value
                hash = hash & 0xFFFFFFFFFFFFFFFF  # to remain as 64bit number
        else:
            for x in range(int(65536/bytesize)):
                buffer = f.read(bytesize)
                (l_value,) = struct.unpack(longlongformat, buffer)
                hash += l_value
                hash = hash & 0xFFFFFFFFFFFFFFFF  # to remain as 64bit number
            f.seek(max(0, filesize-65536), 0)
            for x in range(int(65536/bytesize)):
                buffer = f.read(bytesize)
                (l_value,) = struct.unpack(longlongformat, buffer)
                hash += l_value
                hash = hash & 0xFFFFFFFFFFFFFFFF
        f.close()
        returnedhash = "%016x" % hash
        return returnedhash
    except IOError:
        return "IOError"

def avinfo(filename, cached=True):
    if cached:
        return cache(filename, 'info')
    if os.path.getsize(filename):
        if find_executable('ffprobe'):
            return ffprobe(filename)
        raise EnvironmentError('could to find ffprobe. please install ffmpeg')
    return {'path': filename, 'size': 0}


def ffprobe(filename):
    p = subprocess.Popen([
        cmd('ffprobe'),
        '-show_format',
        '-show_chapters',
        '-show_streams',
        '-print_format',
        'json',
        '-i', filename

    ], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    info, error = p.communicate()
    info = info.decode('utf-8')
    ffinfo = json.loads(info)

    def fix_value(key, value):
        if key == 'r_frame_rate':
            value = value.replace('/', ':')
        elif key == 'bit_rate':
            value = float(value) / 1000
        elif key == 'duration':
            value = float(value)
        elif key == 'size':
            value = int(value)
        return value

    info = {}
    if 'format' not in ffinfo:
        info['error'] = 'badfile'
    else:
        for key in ('duration', 'size', 'bit_rate'):
            if key in ffinfo['format']:
                info[{
                    'bit_rate': 'bitrate'
                }.get(key, key)] = fix_value(key, ffinfo['format'][key])
        info['audio'] = []
        info['video'] = []
        info['metadata'] = ffinfo['format'].get('tags', {})
        chapters = ffinfo.get('chapters', [])
        if chapters:
            info['chapters'] = [
                {
                    'in': float(chapter['start_time']),
                    'out': float(chapter['end_time']),
                    'value': chapter.get('tags', {}).get('title')
                }
                for chapter in chapters if chapter.get('tags', {}).get('title')
            ]
        for s in ffinfo['streams']:
            tags = s.pop('tags', {})
            side_data_list = s.pop('side_data_list', [])
            language = None
            for t in tags:
                if t == 'language':
                    language = tags[t]
                else:
                    info['metadata'][t] = tags[t]
            for kv in side_data_list:
                for k, v in kv.items():
                    info['metadata'][k] = v
            if s.get('codec_type') in ('audio', 'video'):
                stream = {}
                if language and language != 'und':
                    stream['language'] = language
                keys = [ 
                    'codec_name',
                    'width',
                    'height',
                    'bit_rate',
                    'index',
                    'display_aspect_ratio',
                    'sample_rate',
                    'channels',
                ]
                if s['codec_type'] == 'video':
                    keys += [
                        'sample_aspect_ratio',
                        'r_frame_rate',
                        'pix_fmt',
                    ]

                for key in keys:
                    if key in s:
                        stream[{
                            'codec_name': 'codec',
                            'bit_rate': 'bitrate',
                            'index': 'id',
                            'r_frame_rate': 'framerate',
                            'sample_rate': 'samplerate',
                            'pix_fmt': 'pixel_format',
                            'sample_aspect_ratio': 'pixel_aspect_ratio',
                        }.get(key, key)] = fix_value(key, s[key])
                info[s['codec_type']].append(stream)
            elif s.get('codec_type') == 'subtitle':
                info['subtitles'] = info.get('subtitles', [])
                stream = {}
                if language and language != 'und':
                    stream['language'] = language
                for key in (
                    'codec_name',
                    'language',
                    'width',
                    'height',
                ):
                    if key in s:
                        stream[{
                            'codec_name': 'codec',
                        }.get(key, key)] = s[key]
                info['subtitles'].append(stream)
            else:
                pass
                # print s
        for v in info['video']:
            k = 'display_aspect_ratio'
            if 'rotate' in info.get('metadata', {}) and int(info['metadata']['rotate']) in (-90, 90):
                v['width'], v['height'] = v['height'], v['width']
                if k in v:
                    v[k] = ':'.join(reversed(v[k].split(':')))
            elif 'rotation' in info.get('metadata', {}) and int(info['metadata']['rotation']) in (-90, 90):
                v['width'], v['height'] = v['height'], v['width']
                if k in v:
                    v[k] = ':'.join(reversed(v[k].split(':')))
            if k not in v and 'width' in v \
                    or (k in v and v[k] == '0:1'):
                v[k] = '%d:%d' % (v['width'], v['height'])
                v['pixel_aspect_ratio'] = '1:1'
    info['oshash'] = oshash(filename)
    info['path'] = filename
    if 'size' not in info:
        info['size'] = os.path.getsize(filename)

    fix_coverart(info)
    return info

def fix_coverart(info):
    if info.get('video') \
            and info['path'].split('.')[-1] in EXTENSIONS['audio'] \
            and info['video'][0]['codec'] in EXTENSIONS['image'] + ['mjpeg']:
        info['cover'] = info.pop('video')
        info['video'] = []
    return info

def makedirs(path):
    if not os.path.exists(path):
        try:
            os.makedirs(path)
        except OSError as e:
            if e.errno != 17:
                raise

def copy_file(source, target, verbose=False):
    if verbose:
        print('copying', source, 'to', target)
    write_path(target)
    shutil.copyfile(source, target)

def read_file(file, verbose=False):
    if verbose:
        print('reading', file)
    f = open(file, 'rb')
    data = f.read()
    f.close()
    return data

def read_json(file, verbose=False):
    if verbose:
        print('reading', file)
    with open(file) as fd:
        data = json.load(fd)
    return data

def write_file(file, data, verbose=False):
    if verbose:
        print('writing', file)
    if not isinstance(data, bytes):
        data = data.encode('utf-8')
    write_path(file)
    f = open(file, 'wb')
    f.write(data)
    f.close()
    return len(data)

def write_image(file, image, verbose=False):
    if verbose:
        print('writing', file)
    write_path(file)
    image.save(file)

def write_json(file, data, ensure_ascii=True, indent=0, sort_keys=False, verbose=False):
    data = json.dumps(data, ensure_ascii=ensure_ascii, indent=indent, sort_keys=sort_keys)
    write_file(file, data if ensure_ascii else data.encode('utf-8'), verbose=verbose)

def write_link(source, target, verbose=False):
    if verbose:
        print('linking', source, 'to', target)
    write_path(target)
    if os.path.exists(target):
        os.unlink(target)
    os.symlink(source, target)

def write_path(file):
    path = os.path.split(file)[0]
    if path and not os.path.exists(path):
        os.makedirs(path)

def iexists(path):
    parts = path.split(os.sep)
    name = parts[-1].lower()
    if len(parts) == 1:
        folder = '.'
    else:
        folder = os.path.dirname(path)
    try:
        files = os.listdir(folder)
    except FileNotFoundError:
        return False
    files = {os.path.basename(f).lower() for f in files}
    return name in files



# -*- coding: utf-8 -*-
# vi:si:et:sw=4:sts=4:ts=4
from collections import defaultdict

import json
import re
import time
import unicodedata

from urllib.parse import urlencode

from .. import find_re, strip_tags, decode_html
from .. import cache


from . siteparser import SiteParser
from . import duckduckgo
from ..utils import datetime
from ..geo import normalize_country_name, get_country_name


def prepare_url(url, data=None, headers=cache.DEFAULT_HEADERS, timeout=cache.cache_timeout, valid=None, unicode=False):
    headers = headers.copy()
    # https://webapps.stackexchange.com/questions/11003/how-can-i-disable-reconfigure-imdbs-automatic-geo-location-so-it-does-not-defau
    #headers['X-Forwarded-For'] = '72.21.206.80'
    headers['Accept-Language'] = 'en'

    return url, data, headers, timeout, unicode

def read_url(url, data=None, headers=cache.DEFAULT_HEADERS, timeout=cache.cache_timeout, valid=None, unicode=False):
    url, data, headers, timeout, unicode = prepare_url(url, data, headers, timeout, valid, unicode)
    return cache.read_url(url, data, headers, timeout, unicode=unicode)

def delete_url(url, data=None, headers=cache.DEFAULT_HEADERS):
    url, data, headers, timeout, unicode = prepare_url(url, data, headers)
    cache.store.delete(url, data, headers)

def get_url(id):
    return "http://www.imdb.com/title/tt%s/" % id


def reference_section(id):
    return {
        'page': 'reference',
        're': [
            '<h4 name="{id}" id="{id}".*?<table(.*?)</table>'.format(id=id),
            '<a href="/name/.*?>(.*?)</a>'
        ],
        'type': 'list'
    }


def zebra_list(label, more=None):
    conditions = {
        'page': 'reference',
        're': [
            '_label">' + label + '</td>.*?<ul(.*?)</ul>',
            '<li.*?>(.*?)</li>'
        ],
        'type': 'list',
    }
    if more:
        conditions['re'] += more
    return conditions

def zebra_table(label, more=None, type='string'):
    conditions = {
        'page': 'reference',
        're': [
            '_label">' + label + '</td>.*?<td>(.*?)</td>',
        ],
        'type': type,
    }
    if more:
        conditions['re'] += more
    return conditions

def parse_aspectratio(value):
    r = value
    if ':' in value:
        r = value.split(':')
        n = r[0]
        d = r[1].strip().split(' ')[0]
        try:
            if float(d):
                value = str(float(n) / float(d))
            else:
                value = str(float(n))
        except:
            print('failed to parse aspect: %s' % value)
    else:
        value = '.'.join(value.strip().split('.')[:2])
    return value


def technical(label):
    return {
        'page': 'technical',
        're': [
            '<td class="label">\s*?%s\s*?</td>.*?<td>\s*?(.*?)\s*?</td>' % label,
            lambda data: [
                re.sub('\s+', ' ', d.strip()) for d in data.strip().split('<br>')
            ] if data else []
        ],
        'type': 'list'
    }


def tech_spec(metadata):
    tech = {}
    for row in metadata['props']['pageProps']['contentData']['section']['items']:
        title = {
            'aspect ratio': 'aspectratio',
            'sound mix': 'sound',
        }.get(row['rowTitle'].lower(), row['rowTitle'].lower())
        tech[title] = []
        for content in row['listContent']:
            value = content['text']
            tech[title].append(value)
    return tech


def movie_connections(metadata):
    
    connections = {}
    if 'props' not in metadata:
        return connections
    for row in metadata['props']['pageProps']['contentData']['categories']:
        title = {
        }.get(row['name'], row['name'])
        if title not in connections:
            connections[title] = []

        for item in row['section']['items']:
            item_ = {
                'id': item['id'][2:],
            }

            item_['title'] = re.compile('<a.*?>(.*?)</a>').findall(item['listContent'][0]['html'])[0]
            if len(item['listContent']) >=2:
                item_['description'] = strip_tags(item['listContent'][1]['html'])
            connections[title].append(item_)
    return connections


def get_category_by_id(metadata, id):
    for category in metadata['props']['pageProps']['contentData']['categories']:
        if category['id'] == id:
            return category


def get_release_date(metadata):
    releases = get_category_by_id(metadata, 'releases')
    def parse_date(d):
        parsed = None
        for fmt in (
            '%B %d, %Y',
            '%d %B %Y',
            '%B %Y',
        ):
            try:
                parsed = datetime.strptime(d, fmt)
                break
            except:
                pass
        if not parsed:
            return None
        return '%d-%02d-%02d' % (parsed.year, parsed.month, parsed.day)

    dates = []
    for item in releases['section']['items']:
        content = item['listContent'][0]
        date = parse_date(content['text'])
        if date:
            dates.append(date)

    if dates:
        return min(dates)

def get_locations(metadata):
    try:
        locations = [
            row['cardText']
            for row in metadata['props']['pageProps']['contentData']['categories'][0]['section']['items']
        ]
    except:
        locations = []
    return locations


def get_keywords(metadata):
    try:
        keywords = [
            row['rowTitle']
            for row in metadata['props']['pageProps']['contentData']['section']['items']
        ]
    except:
        keywords = []
    return keywords


def get_entity_metadata(metadata):
    data = {}
    entity = metadata['props']['pageProps']['contentData']['entityMetadata']
    data['title'] = entity['titleText']['text']
    data['originalTitle'] = entity['originalTitleText']['text']
    data['year'] = entity['releaseYear']['year']
    data['plot'] = entity['plot']['plotText']['plainText']
    data['country'] = [get_country_name(c['id']) for c in entity['countriesOfOrigin']['countries']]
    data['poster'] = metadata['props']['pageProps']['contentData']['posterData']['image']['url']
    return data


def alternative_titles(metadata):
    titles = defaultdict(list)
    akas = get_category_by_id(metadata, 'akas')

    skip = [
        metadata['props']['pageProps']['contentData']['entityMetadata']['titleText']['text'],
        metadata['props']['pageProps']['contentData']['entityMetadata']['originalTitleText']['text']
    ]
    for row in akas['section']['items']:
        content = row['listContent'][0]
        title = content['text']
        country = row['rowTitle']
        if title in skip:
            continue
        titles[title].append(country)
        #if content.get('subText'):
        #    titles[-1]['subText'] = content['subText']
    return [kv for kv in titles.items()]


'''
'posterIds': {
    'page': 'posters',
    're': '/unknown-thumbnail/media/rm(.*?)/tt',
    'type': 'list'
},
'''

class Imdb(SiteParser):
    '''
    >>> Imdb('0068646')['title'] == 'The Godfather'
    True

    >>> Imdb('0133093')['title'] == 'The Matrix'
    True
    '''
    regex = {
        'alternativeTitles': {
            'page': 'releaseinfo',
            're': [
                '<li role="presentation" class="ipc-metadata-list__item" data-testid="list-item"><button class="ipc-metadata-list-item__label" role="button" tabindex="0" aria-disabled="false">([^>]+)</button.*?<li role="presentation" class="ipc-inline-list__item"><label class="ipc-metadata-list-item__list-content-item"[^>]*?>([^<]+)</label>',
            ],
            'type': 'list'
        },
        'aspectratio': {
            'page': 'reference',
            're': [
                'Aspect Ratio</td>.*?ipl-inline-list__item">\s+([\d\.\:\ ]+)',
                parse_aspectratio,
            ],
            'type': 'float',
        },
        'budget': zebra_table('Budget', more=[
            lambda data: find_re(decode_html(data).replace(',', ''), '\d+')
        ], type='int'),
        'cast': {
            'page': 'reference',
            're': [
                ' <table class="cast_list">(.*?)</table>',
                '<td.*?itemprop="actor".*?>.*?>(.*?)</a>.*?<td class="character">(.*?)</td>',
                lambda ll: [strip_tags(l) for l in ll] if isinstance(ll, list) else strip_tags(ll)
            ],
            'type': 'list'
        },
        'cinematographer': reference_section('cinematographers'),
        'country': zebra_list('Country', more=['<a.*?>(.*?)</a>']),
        'director': reference_section('directors'),
        'editor': reference_section('editors'),
        'composer': reference_section('composers'),
        'episodeTitle': {
            'page': 'reference',
            're': '<h3 itemprop="name">(.*?)<',
            'type': 'string'
        },
        'genre': zebra_list('Genres', more=['<a.*?>(.*?)</a>']),
        'gross': zebra_table('Cumulative Worldwide Gross', more=[
            lambda data: find_re(decode_html(data).replace(',', ''), '\d+')
        ], type='int'),
        'language': zebra_list('Language', more=['<a.*?>(.*?)</a>']),
        'originalTitle': {
            'page': 'releaseinfo',
            're': '<li role="presentation" class="ipc-metadata-list__item" data-testid="list-item"><button class="ipc-metadata-list-item__label" role="button" tabindex="0" aria-disabled="false">\(original title\)</button.*?<li role="presentation" class="ipc-inline-list__item"><label class="ipc-metadata-list-item__list-content-item"[^>]*?>([^<]+)</label>',
            'type': 'string'
        },
        'summary': zebra_table('Plot Summary', more=[
            '<p>(.*?)<em'
        ]),
        'storyline': {
            'page': '',
            're': '<h2>Storyline</h2>.*?<p>(.*?)</p>',
            'type': 'string'
        },
        'posterId': {
            'page': 'reference',
            're': '<img.*?class="titlereference-primary-image".*?src="(.*?)".*?>',
            'type': 'string'
        },
        'producer': reference_section('producers'),
        'productionCompany': {
            'page': 'reference',
            're': [
                'Production Companies.*?<ul(.*?)</ul>',
                '<a href="/company/.*?/">(.*?)</a>'
            ],
            'type': 'list'
        },
        'rating': {
            'page': 'reference',
            're': [
                '<div class="ipl-rating-star ">(.*?)</div>',
                'ipl-rating-star__rating">([\d,.]+?)</span>',
            ],
            'type': 'float'
        },
        #FIXME using some /offsite/ redirect now
        #'reviews': {
        #    'page': 'externalreviews',
        #    're': [
        #        '<ul class="simpleList">(.*?)</ul>',
        #        '<li>.*?<a href="(http.*?)".*?>(.*?)</a>.*?</li>'
        #    ],
        #    'type': 'list'
        #},
        'runtime': zebra_list('Runtime'),
        'color': zebra_list('Color', more=[
            '<a.*?>([^(<]+)',
            lambda r: r[0] if isinstance(r, list) else r,
            strip_tags
        ]),
        'season': {
            'page': 'reference',
            're': [
                '<ul class="ipl-inline-list titlereference-overview-season-episode-numbers">(.*?)</ul>',
                'Season (\d+)',
             ],
            'type': 'int'
        },
        'episode': {
            'page': 'reference',
            're': [
                '<ul class="ipl-inline-list titlereference-overview-season-episode-numbers">(.*?)</ul>',
                'Episode (\d+)',
             ],
            'type': 'int'
        },
        'series': {
            'page': 'reference',
            're': '<h4 itemprop="name">.*?<a href="/title/tt(\d+)',
            'type': 'string'
        },
        'isSeries': {
            'page': 'reference',
            're': 'property=\'og:title\'.*?content=".*?(TV series|TV mini-series).*?"',
            'type': 'string'
        },
        'title': {
            'page': 'releaseinfo',
            're': '<h2.*?>(.*?)</h2>',
            'type': 'string'
        },
        'trivia': {
            'page': 'trivia',
            're': [
                '<div class="sodatext">(.*?)<(br|/div)',
                lambda data: data[0]
            ],
            'type': 'list',
        },
        'votes': {
            'page': 'reference',
            're': [
                'class="ipl-rating-star__total-votes">\((.*?)\)',
                lambda r: r.replace(',', '')
            ],
            'type': 'string'
        },
        'writer': reference_section('writers'),
        'year': {
            'page': 'reference',
            're': [
                '<span class="titlereference-title-year">(.*?)</span>',
                '<a.*?>(\d+)',
            ],
            'type': 'int'
        },
        'credits': {
            'page': 'fullcredits',
            're': [
                lambda data: data.split('<h4'),
                '>(.*?)</h4>.*?(<table.*?</table>)',
                lambda data: [d for d in data if d]
            ],
            'type': 'list'
        },
        'laboratory': technical('Laboratory'),
        'camera': technical('Camera'),
    }

    def read_url(self, url, timeout):
        if self.debug:
            print(url)
        if url not in self._cache:
            self._cache[url] = read_url(url, timeout=timeout, unicode=True)
        return self._cache[url]

    def get_page_data(self, page, timeout=-1):
        url = self.get_url(page)
        data = self.read_url(url, timeout)
        pdata = re.compile('<script id="__NEXT_DATA__" type="application/json">(.*?)</script>', re.DOTALL).findall(data)
        if pdata:
            pdata = pdata[0]
            return json.loads(pdata)
        return {}

    def __init__(self, id, timeout=-1):
        # http://www.imdb.com/help/show_leaf?titlelanguagedisplay
        self.baseUrl = "http://www.imdb.com/title/tt%s/" % id
        self._id = id
        if timeout != 0:
            self._cache = {}
            url = self.baseUrl + 'releaseinfo'
            page = self.read_url(url, timeout=-1)
            if '<h2>See also</h2>' in page:
                timeout = 0
        super(Imdb, self).__init__(timeout)

        url = self.baseUrl + 'reference'
        page = self.read_url(url, timeout=-1)
        if '<title>IMDb: Page not found</title>' in page \
            or 'The requested URL was not found on our server.' in page:
            return
        if "<p>We're sorry, something went wrong.</p>" in page:
            time.sleep(1)
            super(Imdb, self).__init__(0)

        if 'alternativeTitles' in self:
            if len(self['alternativeTitles']) == 2 and \
               isinstance(self['alternativeTitles'][0], str):
               self['alternativeTitles'] = [self['alternativeTitles']]

        for key in ('country', 'genre', 'language', 'sound', 'color'):
            if key in self:
                self[key] = [x[0] if len(x) == 1 and isinstance(x, list) else x for x in self[key]]
                self[key] = list(filter(lambda x: x.lower() != 'home', self[key]))

        #normalize country names
        if 'country' in self:
            self['country'] = [normalize_country_name(c) or c for c in self['country']]


        def cleanup_title(title):
            if isinstance(title, list):
                title = title[0]
            if title.startswith('"') and title.endswith('"'):
                title = title[1:-1]
            if title.startswith("'") and title.endswith("'"):
                title = title[1:-1]
            title = re.sub('\(\#[.\d]+\)', '', title)
            return title.strip()

        for t in ('title', 'originalTitle'):
            if t in self:
                self[t] = cleanup_title(self[t])

        if 'alternativeTitles' in self:
            alt = {}
            for t in self['alternativeTitles']:
                if t[0].strip() in ('World-wide (English title)', ):
                    self['title'] = cleanup_title(t[1])
            for t in self['alternativeTitles']:
                title = cleanup_title(t[1])
                if title.lower() not in (self.get('title', '').lower(), self.get('originalTitle', '').lower()):
                    if title not in alt:
                        alt[title] = []
                    for c in t[0].split('/'):
                        for cleanup in ('International', '(working title)', 'World-wide'):
                            c = c.replace(cleanup, '')
                        c = c.split('(')[0].strip()
                        if c:
                            alt[title].append(c)
            self['alternativeTitles'] = []
            for t in sorted(alt, key=lambda a: sorted(alt[a])):
                countries = sorted(set([normalize_country_name(c) or c for c in alt[t]]))
                self['alternativeTitles'].append((t, countries))
            if not self['alternativeTitles']:
                del self['alternativeTitles']

        if 'runtime' in self and self['runtime']:
            if isinstance(self['runtime'], list):
                self['runtime'] = self['runtime'][0]
            if 'min' in self['runtime']:
                base = 60
            else:
                base = 1
            self['runtime'] = int(find_re(self['runtime'], '([0-9]+)')) * base
        if 'runtime' in self and not self['runtime']:
            del self['runtime']

        if 'sound' in self:
            self['sound'] = list(sorted(set(self['sound'])))

        if 'cast' in self:
            if isinstance(self['cast'][0], str):
                self['cast'] = [self['cast']]
            self['actor'] = [c[0] for c in self['cast']]
            def cleanup_character(c):
                c = c.replace('(uncredited)', '').strip()
                c = re.sub('\s+', ' ', c)
                return c
            self['cast'] = [{'actor': x[0], 'character': cleanup_character(x[1])}
                            for x in self['cast']]


        if 'isSeries' in self:
            del self['isSeries']
            self['isSeries'] = True
        if 'episodeTitle' in self:
            self['episodeTitle'] = re.sub('Episode \#\d+\.\d+', '', self['episodeTitle'])


        #make lists unique but keep order
        for key in ('director', 'language'):
            if key in self:
                self[key] = [x for i,x in enumerate(self[key])
                             if x not in self[key][i+1:]]

        for key in ('actor', 'writer', 'producer', 'editor', 'composer'):
            if key in self:
                if isinstance(self[key][0], list):
                    self[key] = [i[0] for i in self[key] if i]
                self[key] = sorted(list(set(self[key])), key=lambda a: self[key].index(a))


        if 'budget' in self and 'gross' in self:
            self['profit'] = self['gross'] - self['budget']

        metadata = self.get_page_data('releaseinfo')
        releasedate = get_release_date(metadata)
        if releasedate:
            self['releasedate'] = releasedate

        metadata = self.get_page_data('keywords')
        keywords = get_keywords(metadata)
        if keywords:
            self['keyword'] = keywords

        metadata = self.get_page_data('locations')
        locations = get_locations(metadata)
        if locations:
            self['filmingLocations'] = locations

        if 'summary' not in self and 'storyline' in self:
            self['summary'] = self.pop('storyline')
        if 'summary' in self:
            if isinstance(self['summary'], list):
                self['summary'] = self['summary'][0]
            self['summary'] = strip_tags(self['summary'].split('</p')[0]).split('  Written by\n')[0].strip()
        else:

            try:
                summary = metadata['props']['pageProps']['contentData']['entityMetadata']['plot']['plotText']['plainText']
                self['summary'] = summary

            except:
                pass

        #self['connections'] = movie_connections(self.get_page_data('movieconnections'))
        self['connections'] = self._get_connections()

        spec = tech_spec(self.get_page_data('technical'))
        for key in spec:
            if not self.get(key):
                self[key] = spec[key]

        if 'credits' in self:
            credits = [
                [
                    strip_tags(d[0].replace(' by', '')).strip(),
                    [
                        [
                            strip_tags(x[0]).strip(),
                            [t.strip().split(' (')[0].strip() for t in x[2].split(' / ')]
                        ]
                        for x in
                        re.compile('<td class="name">(.*?)</td>.*?<td>(.*?)</td>.*?<td class="credit">(.*?)</td>', re.DOTALL).findall(d[1])
                    ]
                ] for d in self['credits'] if d
            ]
            credits = [c for c in credits if c[1]]

            self['credits'] = []
            self['lyricist'] = []
            self['singer'] = []
            for department, crew in credits:
                department = department.replace('(in alphabetical order)', '').strip()
                for c in crew:
                    name = c[0]
                    roles = c[1]
                    self['credits'].append({
                        'name': name,
                        'roles': roles,
                        'deparment': department
                    })
                    if department == 'Music Department':
                        if 'lyricist' in roles:
                            self['lyricist'].append(name)
                        if 'playback singer' in roles:
                            self['singer'].append(name)
            if not self['credits']:
                del self['credits']

        if 'credits' in self:
            for key, deparment in (
                ('director', 'Series Directed'),
                ('writer', 'Series Writing Credits'),
                ('cinematographer', 'Series Cinematography'),
            ):
                if key not in self:
                    series_credit = [c for c in self['credits'] if c.get('deparment') == deparment]
                    if series_credit:
                        self[key] = [c['name'] for c in series_credit]
        creator = []
        for c in self.get('credits', []):
            if '(created by)' in c['roles'] and c['name'] not in creator:
                creator.append(c['name'])
            if '(creator)' in c['roles'] and c['name'] not in creator:
                creator.append(c['name'])
        if creator:
            self['creator'] = creator

        if 'series' in self:
            series = Imdb(self['series'], timeout=timeout)
            self['seriesTitle'] = series['title']
            if 'episodeTitle' in self:
                self['seriesTitle'] = series['title']
                if 'season' in self and 'episode' in self:
                    self['title'] = "%s (S%02dE%02d) %s" % (
                        self['seriesTitle'], self['season'], self['episode'], self['episodeTitle'])
                else:
                    self['title'] = "%s (S01) %s" % (self['seriesTitle'], self['episodeTitle'])
                    self['season'] = 1
                self['title'] = self['title'].strip()
            if 'director' in self:
                self['episodeDirector'] = self['director']

            if 'creator' not in series and 'director' in series:
                series['creator'] = series['director']
                if len(series['creator']) > 10:
                    series['creator'] = series['director'][:1]

            for key in ['creator', 'country']:
                if key in series:
                    self[key] = series[key]

            if 'year' in series:
                self['seriesYear'] = series['year']
                if 'year' not in self:
                    self['year'] = series['year']

            if 'year' in self:
                self['episodeYear'] = self['year']
            if 'creator' in self:
                self['seriesDirector'] = self['creator']
            if 'originalTitle' in self:
                del self['originalTitle']
        else:
            for key in ('seriesTitle', 'episodeTitle', 'season', 'episode'):
                if key in self:
                    del self[key]
        if 'creator' in self:
            if 'director' in self:
                self['episodeDirector'] = self['director']
            self['director'] = self['creator']

    def _get_connections(self):
        query = '''query {
    title(id: "tt%s") {
        id
        titleText {
           text
        }
        connections(first: 5000) {
            edges {
                node {
                    associatedTitle {
                        id
                        titleText {
                            text
                        }
                    }
                    category {
                        text
                    }
                    text
                }
            }
        }
    }
}
''' % self._id
        url = 'https://caching.graphql.imdb.com/'
        headers = cache.DEFAULT_HEADERS.copy()
        headers.update({
             'Accept': 'application/graphql+json, application/json',
             'Origin': 'https://www.imdb.com',
             'Referer': 'https://www.imdb.com',
             'x-imdb-user-country': 'US',
             'x-imdb-user-language': 'en-US',
             'content-type': 'application/json',
             'Accept-Language': 'en,en-US;q=0.5'
        })
        #response = requests.post(url, json=
        response = json.loads(read_url(url, data=json.dumps({
            "query": query
        }), headers=headers))
        connections = {}
        for c in response['data']['title']['connections']['edges']:
            cat = c['node']['category']['text']
            if cat not in connections:
                connections[cat] = []
            connection = {
                'id': c['node']['associatedTitle']['id'][2:],
                'title': c['node']['associatedTitle']['titleText']['text'],
            }
            description = c['node'].get('text', '')
            if description:
                connection['description'] = description
            connections[cat].append(connection)
        return connections


class ImdbCombined(Imdb):
    def __init__(self, id, timeout=-1):
        _regex = {}
        for key in self.regex:
            if self.regex[key]['page'] in ('releaseinfo', 'reference'):
                _regex[key] = self.regex[key]
        self.regex = _regex
        super(ImdbCombined, self).__init__(id, timeout)

def get_movie_by_title(title, timeout=-1):
    '''
    This only works for exact title matches from the data dump
    Usually in the format
        Title (Year)
        "Series Title" (Year) {(#Season.Episode)}
        "Series Title" (Year) {Episode Title (#Season.Episode)}

    If there is more than one film with that title for the year
        Title (Year/I)

    >>> str(get_movie_by_title(u'"Father Knows Best" (1954) {(#5.34)}'))
    '1602860'

    >>> str(get_movie_by_title(u'The Matrix (1999)'))
    '0133093'

    >>> str(get_movie_by_title(u'Little Egypt (1951)'))
    '0043748'

    >>> str(get_movie_by_title(u'Little Egypt (1897/I)'))
    '0214882'

    >>> get_movie_by_title(u'Little Egypt')
    None 

    >>> str(get_movie_by_title(u'"Dexter" (2006) {Father Knows Best (#1.9)}'))
    '0866567'
    '''
    params = {'s': 'tt', 'q': title}
    if not isinstance(title, bytes):
        try:
            params['q'] = unicodedata.normalize('NFKC', params['q']).encode('latin-1')
        except:
            params['q'] = params['q'].encode('utf-8')
    params = urlencode(params)
    url = "http://www.imdb.com/find?" + params
    data = read_url(url, timeout=timeout, unicode=True)
    #if search results in redirect, get id of current page
    r = '<meta property="og:url" content="http://www.imdb.com/title/tt(\d+)/" />'
    results = re.compile(r).findall(data)    
    if results:
        return results[0]
    return None
 
def get_movie_id(title, director='', year='', timeout=-1):
    '''
    >>> str(get_movie_id('The Matrix'))
    '0133093'

    >>> str(get_movie_id('2 or 3 Things I Know About Her', 'Jean-Luc Godard'))
    '0060304'

    >>> str(get_movie_id('2 or 3 Things I Know About Her', 'Jean-Luc Godard', '1967'))
    '0060304'

    >>> str(get_movie_id(u"Histoire(s) du cinema: Le controle de l'univers", u'Jean-Luc Godard'))
    '0179214'

    >>> str(get_movie_id(u"Histoire(s) du cinéma: Le contrôle de l'univers", u'Jean-Luc Godard'))
    '0179214'

    '''
    imdbId = {
        (u'Le jour se l\xe8ve', u'Marcel Carn\xe9'): '0031514',
        (u'Wings', u'Larisa Shepitko'): '0061196',
        (u'The Ascent', u'Larisa Shepitko'): '0075404',
        (u'Fanny and Alexander', u'Ingmar Bergman'): '0083922',
        (u'Torment', u'Alf Sj\xf6berg'): '0036914',
        (u'Crisis', u'Ingmar Bergman'): '0038675',
        (u'To Joy', u'Ingmar Bergman'): '0043048',
        (u'Humain, trop humain', u'Louis Malle'): '0071635',
        (u'Place de la R\xe9publique', u'Louis Malle'): '0071999',
        (u'God\u2019s Country', u'Louis Malle'): '0091125',
        (u'Flunky, Work Hard', u'Mikio Naruse'): '0022036',
        (u'The Courtesans of Bombay', u'Richard Robbins') : '0163591',
        (u'Je tu il elle', u'Chantal Akerman') : '0071690',
        (u'Hotel Monterey', u'Chantal Akerman') : '0068725',
        (u'No Blood Relation', u'Mikio Naruse') : '023261',
        (u'Apart from You', u'Mikio Naruse') : '0024214',
        (u'Every-Night Dreams', u'Mikio Naruse') : '0024793',
        (u'Street Without End', u'Mikio Naruse') : '0025338',
        (u'Sisters of the Gion', u'Kenji Mizoguchi') : '0027672',
        (u'Osaka Elegy', u'Kenji Mizoguchi') : '0028021',
        (u'Blaise Pascal', u'Roberto Rossellini') : '0066839',
        (u'Japanese Girls at the Harbor', u'Hiroshi Shimizu') : '0160535',
        (u'The Private Life of Don Juan', u'Alexander Korda') : '0025681',
        (u'Last Holiday', u'Henry Cass') : '0042665',
        (u'A Colt Is My Passport', u'Takashi  Nomura') : '0330536',
        (u'Androcles and the Lion', u'Chester Erskine') : '0044355',
        (u'Major Barbara', u'Gabriel Pascal') : '0033868',
        (u'Come On Children', u'Allan King') : '0269104',

        (u'Jimi Plays Monterey & Shake! Otis at Monterey', u'D. A. Pennebaker and Chris Hegedus') : '',
        (u'Martha Graham: Dance on Film', u'Nathan Kroll') : '',
        (u'Carmen', u'Carlos Saura'): '0085297',
        (u'The Story of a Cheat', u'Sacha Guitry'): '0028201',
        (u'Weekend', 'Andrew Haigh'): '1714210',
    }.get((title, director), None)
    if imdbId:
        return imdbId
    params = {'s': 'tt', 'q': title}
    if director:
        params['q'] = u'"%s" %s' % (title, director)
    if year:
        params['q'] = u'"%s (%s)" %s' % (title, year, director)
    google_query = "site:imdb.com %s" % params['q']
    if not isinstance(params['q'], bytes):
        try:
            params['q'] = unicodedata.normalize('NFKC', params['q']).encode('latin-1')
        except:
            params['q'] = params['q'].encode('utf-8')
    params = urlencode(params)
    url = "http://www.imdb.com/find?" + params
    #print url

    data = read_url(url, timeout=timeout, unicode=True)
    #if search results in redirect, get id of current page
    r = '<meta property="og:url" content="http://www.imdb.com/title/tt(\d+)/" />'
    results = re.compile(r).findall(data)    
    if results:
        return results[0]
    #otherwise get first result
    r = '<td valign="top">.*?<a href="/title/tt(\d+)/"'
    results = re.compile(r).findall(data)
    if results:
        return results[0]

    #print((title, director), ": '',")
    #print(google_query)
    #results = google.find(google_query, timeout=timeout)
    results = duckduckgo.find(google_query, timeout=timeout)
    if results:
        for r in results[:2]:
            imdbId = find_re(r[1], 'title/tt(\d+)')
            if imdbId:
                return imdbId
    #or nothing
    return ''

def get_movie_poster(imdbId):
    '''
    >>> get_movie_poster('0133093')
    'http://ia.media-imdb.com/images/M/MV5BMjEzNjg1NTg2NV5BMl5BanBnXkFtZTYwNjY3MzQ5._V1._SX338_SY475_.jpg'
    '''
    info = ImdbCombined(imdbId)
    if 'posterId' in info:
        poster = info['posterId']
        if '@._V' in poster:
            poster = poster.split('@._V')[0] + '@.jpg'
        return poster
    elif 'series' in info:
        return get_movie_poster(info['series'])
    return ''

def get_episodes(imdbId, season=None):
    episodes = {}
    url = 'http://www.imdb.com/title/tt%s/episodes' % imdbId
    if season:
        url += '?season=%d' % season
        data = cache.read_url(url).decode()
        for e in re.compile('<div data-const="tt(\d+)".*?>.*?<div>S(\d+), Ep(\d+)<\/div>\n<\/div>', re.DOTALL).findall(data):
            episodes['S%02dE%02d' % (int(e[1]), int(e[2]))] = e[0]
    else:
        data = cache.read_url(url)
        match = re.compile('<strong>Season (\d+)</strong>').findall(data)
        if match:
            for season in range(1, int(match[0]) + 1):
               episodes.update(get_episodes(imdbId, season))
    return episodes

def max_votes():
    url = 'http://www.imdb.com/search/title?num_votes=500000,&sort=num_votes,desc'
    data = cache.read_url(url).decode('utf-8', 'ignore')
    votes = max([
        int(v.replace(',', ''))
        for v in re.compile('<span name="nv" data-value="(\d+)"').findall(data)
    ])
    return votes

def guess(title, director='', timeout=-1):
    return get_movie_id(title, director, timeout=timeout)

if __name__ == "__main__":
    import json
    print(json.dumps(Imdb('0306414'), indent=2))
    #print json.dumps(Imdb('0133093'), indent=2)




# -*- coding: utf-8 -*-
# vi:si:et:sw=4:sts=4:ts=4
# GPL 2011
from __future__ import print_function
from types import MethodType
import gzip
import mimetypes
import os
import shutil
import sys
import time

from http import cookiejar as cookielib
from io import BytesIO
import urllib
from urllib.parse import urlparse
import requests

from . import __version__
from .utils import json

__all__ = ['getAPI', 'API']


CHUNK_SIZE = 1024*1024*5

def getAPI(url, cj=None):
    return API(url, cj)

class API(object):
    __version__ = __version__
    __name__ = 'ox'
    DEBUG = False
    debuglevel = 0

    def __init__(self, url, cj=None):
        if cj:
            self._cj = cj
        else:
            self._cj = cookielib.CookieJar()

        self._requests_session = requests.Session()
        self._requests_session.cookies = self._cj
        self._requests_session.headers = {
            'User-Agent': '%s/%s' % (self.__name__, self.__version__),
            'Accept-Encoding': 'gzip, deflate',
        }
        self.url = url
        r = self._request('api', {'docs': True})
        self._properties = r['data']['actions']
        self._actions = r['data']['actions'].keys()
        for a in r['data']['actions']:
            self._add_action(a)

    def _add_method(self, method, name):
        if name is None:
            name = method.func_name
        setattr(self, name, MethodType(method, self))

    def _add_action(self, action):
        def method(self, *args, **kw):
            if args and kw:
                raise ValueError('pass either a dictionary or kwargs, not both')
            if not kw:
                if args:
                    kw = args[0]
                else:
                    kw = None
            return self._request(action, kw)
        if 'doc' in self._properties[action]:
            method.__doc__ = self._properties[action]['doc']
        method.func_name = action
        self._add_method(method, action)

    def _json_request(self, url, data, files=None):
        result = {}
        try:
            request = self._requests_session.post(url, data=data, files=files)
            result = request.json()
            return result
        except urllib.error.HTTPError as e:
            if self.DEBUG:
                import webbrowser
                if e.code >= 500:
                    with open('/tmp/error.html', 'wb') as f:
                        f.write(e.read())
                    webbrowser.open_new_tab('/tmp/error.html')

            result = e.read()
            try:
                result = result.decode('utf-8')
                result = json.loads(result)
            except:
                result = {'status': {}}
            result['status']['code'] = e.code
            result['status']['text'] = str(e)
            return result
        except:
            if self.DEBUG:
                import webbrowser
                import traceback
                traceback.print_exc()
                if result:
                    with open('/tmp/error.html', 'w') as f:
                        f.write(str(result))
                    webbrowser.open_new_tab('/tmp/error.html')
            raise

    def _request(self, action, data=None):
        form = {
            'action': action
        }
        if data:
            form['data'] = json.dumps(data)
        return self._json_request(self.url, form)

    def get_url(self, url):
        return self._requests_session.get(url).content

    def save_url(self, url, filename, overwrite=False):
        chunk_size = 16 * 1024
        if not os.path.exists(filename) or overwrite:
            dirname = os.path.dirname(filename)
            if dirname and not os.path.exists(dirname):
                os.makedirs(dirname)
            tmpname = filename + '.tmp'
            with open(tmpname, 'wb') as fd:
                r = self._requests_session.get(url)
                for chunk in iter(lambda: r.read(chunk_size), b''):
                    fd.write(chunk)
            shutil.move(tmpname, filename)

    def upload_chunks(self, url, filename, data=None, silent=False):
        data = self._json_request(url, data)

        def full_url(path):
            if path.startswith('/'):
                u = urlparse(url)
                path = '%s://%s%s' % (u.scheme, u.netloc, path)
            return path

        if 'uploadUrl' in data:
            uploadUrl = full_url(data['uploadUrl'])
            f = open(filename, 'rb')
            fsize = os.stat(filename).st_size
            done = 0
            if 'offset' in data and data['offset'] < fsize:
                done = data['offset']
                f.seek(done)
                resume_offset = done
            else:
                resume_offset = 0
            chunk = f.read(CHUNK_SIZE)
            fname = os.path.basename(filename)
            mime_type = mimetypes.guess_type(fname)[0] or 'application/octet-stream'
            if not isinstance(fname, bytes):
                fname = fname.encode('utf-8')
            while chunk:
                meta = {
                    'offset': str(done)
                }
                if len(chunk) < CHUNK_SIZE or f.tell() == fsize:
                    meta['done'] = '1'
                files = [
                    ('chunk', (fname, chunk, mime_type))
                ]
                try:
                    data = self._json_request(uploadUrl, meta, files=files)
                except KeyboardInterrupt:
                    if not slient:
                        print("\ninterrupted by user.")
                    sys.exit(1)
                except:
                    if not slient:
                        print("uploading chunk failed, will try again in 5 seconds\r", end='')
                    sys.stdout.flush()
                    data = {'result': -1}
                    time.sleep(5)
                if data and 'status' in data:
                    if data['status']['code'] == 403:
                        if not slient:
                            print("login required")
                        return False
                    if data['status']['code'] != 200:
                        if not slient:
                            print("request returned error, will try again in 5 seconds")
                        if self.DEBUG:
                            print(data)
                        time.sleep(5)
                if data and data.get('result') == 1:
                    done += len(chunk)
                    if data.get('offset') not in (None, done):
                        if not slient:
                            print('server offset out of sync, continue from', data['offset'])
                        done = data['offset']
                        f.seek(done)
                    chunk = f.read(CHUNK_SIZE)
            if data and 'result' in data and data.get('result') == 1:
                return data.get('id', True)
            else:
                return False
        return False

def signin(url):
    import sys
    from getpass import getpass
    from .web import auth

    if not url.startswith('http'):
        site = url
        url = 'https://%s/api/' % url
    else:
        site = url.split('/')[2]
    if not url.endswith('/'):
        url += '/'
    api = API(url)
    update = False
    try:
        credentials = auth.get(site)
    except:
        credentials = {}
        print('Please provide your username and password for %s:' % site)
        credentials['username'] = input('Username: ')
        credentials['password'] = getpass('Password: ')
        update = True
    r = api.signin(**credentials)
    if 'errors' in r.get('data', {}):
        for kv in r['data']['errors'].items():
            print('%s: %s' % kv)
        sys.exit(1)
    if update:
        auth.update(site, credentials)
    return api




# -*- coding: utf-8 -*-
# vi:si:et:sw=4:sts=4:ts=4
import re

import urllib
import ox
from ox import strip_tags, decode_html
from ox.cache import read_url
import lxml.html


def find(query, timeout=ox.cache.cache_timeout):
    """
    Returns tuples with title, url, description
    """
    if not isinstance(query, bytes):
        query = query.encode('utf-8')
    params = urllib.parse.urlencode({'q': query})
    url = 'http://duckduckgo.com/html/?' + params
    data = read_url(url, timeout=timeout).decode('utf-8')
    doc = lxml.html.document_fromstring(data)
    results = []
    for e in doc.xpath("//a[contains(@class, 'result__a')]"):
        url = e.attrib['href']
        if 'uddg=' in url:
            url = urllib.parse.unquote(url.split('&uddg=')[-1])
        title = e.text_content()
        description = ''
        results.append((title, url, description))
    return results



# -*- coding: utf-8 -*-
# vi:si:et:sw=4:sts=4:ts=4
import math


def center(lat_sw, lng_sw, lat_ne=None, lng_ne=None):
    if not lat_ne and not lng_ne:
        return min(lat_sw, lng_sw) + abs(lat_sw-lng_sw)/2
    else:
        return (center(lat_sw,lng_sw), center(lat_ne, lng_ne))

def area(lat_sw, lng_sw, lat_ne, lng_ne):
    return (lat_ne - lat_sw) * (lng_ne - lng_sw)

def latlngspan2latlng(lat, lng, latSpan, lngSpan):
    return dict(
        lat_sw = lat - latSpan, lng_sw = lng - lngSpan,
        lat_ne = lat + latSpan, lng_ne = lng + latSpan
    )

def parse_location_string(location_string):
    l = location_string.split('+')
    if len(l) == 1:
        l = location_string.split(';')
    l = [i.strip() for i in l]
    l = filter(lambda x: x, l)
    return l




#!/usr/bin/python
# -*- coding: utf-8 -*-
# vi:si:et:sw=4:sts=4:ts=4

from __future__ import division

try:
    from PIL import Image
    from PIL import ImageDraw
    from PIL import ImageFont
except:
    import Image
    import ImageDraw
    import ImageFont


ZONE_INDEX = [
    (int(x / 2) + int(y / 4) * 4)
    for x, y in [
        (pixel_index % 8, int(pixel_index / 8))
        for pixel_index in range(64)
    ]
]


def textsize(draw, text, font):
    left, top, right, bottom = draw.textbbox((0, 0), text, font=font)
    return (right, bottom)


def drawText(image, position, text, font_file, font_size, color):
    draw = ImageDraw.Draw(image)
    font = ImageFont.truetype(font_file, font_size, encoding='unic')
    draw.text(position, text, fill=color, font=font)
    size = textsize(draw, text, font)
    version = getattr(Image, 'PILLOW_VERSION', None)
    if version and version > '2.1.0' and version < '2.6.1':
        offset = font.getoffset(text)
    else:
        offset = (0, 0)
    return (size[0] + offset[0], size[1] + offset[1])

def getHSL(rgb):
    rgb = [x / 255 for x in rgb]
    maximum = max(rgb)
    minimum = min(rgb)
    hsl = [0.0, 0.0, 0.0]
    hsl[2] = (maximum + minimum) / 2
    if maximum == minimum:
        hsl[0] = 0.0
        hsl[1] = 0.0
    else:
        if maximum == rgb[0]:
            hsl[0] = (60 * (rgb[1] - rgb[2]) / (maximum - minimum) + 360) % 360
        elif maximum == rgb[1]:
            hsl[0] = 60 * (rgb[2] - rgb[0]) / (maximum - minimum) + 120
        else:
            hsl[0] = 60 * (rgb[0] - rgb[1]) / (maximum - minimum) + 240
        if hsl[2] <= 0.5:
            hsl[1] = (maximum - minimum) / (2 * hsl[2])
        else:
            hsl[1] = (maximum - minimum) / (2 - 2 * hsl[2])
    return tuple(hsl)

def getImageHash(image_file, mode):
    image = Image.open(image_file).convert('RGB').resize((8, 8), Image.LANCZOS)
    image_hash = 0
    if mode == 'color':
        # divide the image into 8 zones:
        # 0 0 1 1 2 2 3 3
        # 0 0 1 1 2 2 3 3
        # 0 0 1 1 2 2 3 3
        # 0 0 1 1 2 2 3 3
        # 4 4 5 5 6 6 7 7
        # 4 4 5 5 6 6 7 7
        # 4 4 5 5 6 6 7 7
        # 4 4 5 5 6 6 7 7
        image_data = image.getdata()
        zone_values = [[] for i in range(8)]
        for pixel_index, pixel_value in enumerate(image_data):
            zone_values[ZONE_INDEX[pixel_index]].append(pixel_value)
        for zone_index, pixel_values in enumerate(zone_values):
            # get the mean for each color channel
            mean = [int(round(sum(x) / 8)) for x in zip(*pixel_values)]
            # store the mean color of each zone as an 8-bit value:
            # RRRGGGBB
            color_index = sum((
                int(mean[0] / 32) << 5,
                int(mean[1] / 32) << 2,
                int(mean[2] / 64)
            ))
            image_hash += color_index * pow(2, zone_index * 8)
    elif mode == 'shape':
        # pixels brighter than the mean register as 1,
        # pixels equal to or darker than the mean as 0
        image_data = image.convert('L').getdata()
        image_mean = sum(image_data) / 64
        for pixel_index, pixel_value in enumerate(image_data):
            if pixel_value > image_mean:
                image_hash += pow(2, pixel_index)
    image_hash = hex(image_hash)[2:].upper()
    if image_hash.endswith('L'):
        image_hash = image_hash[:-1]
    image_hash = '0' * (16 - len(image_hash)) + image_hash
    return image_hash

def getImageHeat(image_file):
    image = Image.open(image_file).convert('RGB').resize((16, 16), Image.LANCZOS)
    pixel = image.load()
    image_heat = 0
    for y in range(image.size[1]):
        for x in range(image.size[0]):
            pixel_heat = []
            for y_ in range(max(y - 1, 0), min(y + 2, image.size[1])):
                for x_ in range(max(x - 1, 0), min(x + 2, image.size[0])):
                    if x != x_ or y != y_:
                        for c in range(3):
                            pixel_heat.append(abs(pixel[x, y][c] - pixel[x_, y_][c]))
            image_heat += sum(pixel_heat) / len(pixel_heat)
    return image_heat / 256

def getImageHSL(image_file):
    image = Image.open(image_file).convert('RGB').resize((1, 1), Image.LANCZOS)
    return getHSL(image.getpixel((0, 0)))

def getRGB(hsl):
    hsl = list(hsl)
    hsl[0] /= 360
    rgb = [0, 0, 0]
    if hsl[1] == 0:
        rgb = [hsl[2], hsl[2], hsl[2]]
    else:
        if hsl[2] < 1/2:
            v2 = hsl[2] * (1 + hsl[1])
        else:
            v2 = hsl[1] + hsl[2] - (hsl[1] * hsl[2])
        v1 = 2 * hsl[2] - v2
        for i in range(3):
            v3 = hsl[0] + (1 - i) * 1/3;
            if v3 < 0:
                v3 += 1
            elif v3 > 1:
                v3 -= 1
            if v3 < 1/6:
                rgb[i] = v1 + ((v2 - v1) * 6 * v3)
            elif v3 < 1/2:
                rgb[i] = v2
            elif v3 < 2/3:
                rgb[i] = v1 + ((v2 - v1) * 6 * (2/3 - v3))
            else:
                rgb[i] = v1
    return tuple([int(x * 255) for x in rgb])

def getTextSize(image, text, font_file, font_size):
    draw = ImageDraw.Draw(image)
    font = ImageFont.truetype(font_file, font_size, encoding='unic')
    size = textsize(draw, text, font)
    version = getattr(Image, 'PILLOW_VERSION', None)
    if version and version > '2.1.0' and version < '2.6.1':
        offset = font.getoffset(text)
    else:
        offset = (0, 0)
    return (size[0] + offset[0], size[1] + offset[1])

def wrapText(text, max_width, max_lines, font_file, font_size):
    # wraps text to max_width and max_lines
    def get_min_width():
        # returns the width of the longest non-hyphenated word
        min_width = 0
        for word in words:
            width = get_width(word)
            if width <= max_width and width > min_width:
                min_width = width
        return min_width

    def get_width(string):
        return textsize(draw, string, font)[0]

    image = Image.new('RGB', (1, 1))
    draw = ImageDraw.Draw(image)
    font = ImageFont.truetype(font_file, font_size, encoding='unic')
    ellipsis = u'…'
    separators = ['-', '+', '/', ':']
    if get_width(text) <= max_width:
        # text fits in one line
        lines = [text]
    else:
        lines = ['']
        words = []
        spaces = []
        test_words = text.split(' ')
        for word in test_words:
            if get_width(word) <= max_width:
                # word fits in one line
                words.append(word)
                spaces.append(' ')
            else:
                # word does not fit in one line
                position = 0
                test_word = word
                for separator in separators:
                    test_word = test_word.replace(separator, ' ')
                parts = test_word.split(' ')
                for i, part in enumerate(parts):
                    words.append(part)
                    if i < len(parts) - 1:
                        position += len(part) + 1
                        spaces.append(word[position - 1])
                    else:
                        spaces.append(' ')
        if max_lines:
            # test if the same number of lines can be achieved with shorter
            # lines, without hyphenating words that are not yet hyphenated
            best_lines = len(wrapText(text, max_width, 0, font_file, font_size))
            test_lines = best_lines
            min_width = get_min_width()
            while test_lines == best_lines and max_width >= min_width:
                max_width -= 1
                test_lines = len(wrapText(text, max_width, 0, font_file, font_size))
            max_width += 1
        for i, word in enumerate(words):
            line = len(lines) - 1
            word_width = get_width(word)
            if word_width <= max_width:
                # word fits in one line
                test = (lines[line] + word + spaces[i]).strip()
                if get_width(test) <= max_width:
                    # word fits in current line
                    lines[line] = test + (' ' if spaces[i] == ' ' else '')
                elif max_lines == 0 or line < max_lines - 1:
                    # word fits in next line
                    lines.append(word + spaces[i])
                else:
                    # word does not fit in last line
                    test = lines[line].strip() + ellipsis
                    if get_width(test) <= max_width:
                        # ellipsis fits in last line
                        lines[line] = test
                    else:
                        # ellipsis does not fit in last line
                        test_words = lines[line].split(' ')
                        while get_width(test) > max_width:
                            test_words.pop()
                            test = ' '.join(test_words) + ellipsis
                        if test == ellipsis:
                            # ellipsis does not fit after first word of last line
                            test = lines[line][:-1] + ellipsis
                            while get_width(test) > max_width:
                                test = test[:-2] + ellipsis
                        lines[line] = test
                    break
            else:
                # word does not fit in one line
                chars = list(word)
                for char in chars:
                    line = len(lines) - 1
                    test = (lines[line] + char + '-').strip()
                    if get_width(test) <= max_width:
                        # char fits in current line
                        lines[line] = test[:-1]
                    elif max_lines == 0 or line < max_lines - 1:
                        # char fits in next line
                        if test[-3] == ' ':
                            lines[line] = test[:-3]
                        else:
                            lines[line] = test[:-2] + '-'
                        lines.append(char)
                    else:
                        # char does not fit in last line
                        test = lines[line] + char + ellipsis
                        while get_width(test) > max_width:
                            test = test[:-2] + ellipsis
                        lines[line] = test
                lines[line] += ' '
    return lines



# -*- coding: utf-8 -*-
# vi:si:et:sw=4:sts=4:ts=4
from __future__ import print_function

from ox.cache import read_url
from ox.html import decode_html
from ox.text import find_re


def get_lyrics(title, artist):
    html = read_url('http://lyricsfly.com/api/')
    key = find_re(html, '<font color=green><b>(.*?)</b></font>')
    url = 'http://lyricsfly.com/api/api.php?i=%s&a=%s&t=%s' % (key, artist, title)
    xml = read_url(url)
    lyrics = find_re(xml, '<tx>(.*?)\[br\] Lyrics [a-z]* by lyricsfly.com')
    lyrics = lyrics.replace('\n', '').replace('\r', '')
    lyrics = lyrics.replace('[br]', '\n').strip()
    lyrics.replace('\n\n\n', '\n\n')
    lyrics = decode_html(lyrics.replace('&amp;', '&'))
    return lyrics

if __name__ == '__main__':
    print(get_lyrics('Election Day', 'Arcadia'))



# -*- coding: UTF-8 -*-
# vi:si:et:sw=4:sts=4:ts=4
from __future__ import print_function
import re

import ox.cache
from ox.cache import read_url
from ox.html import strip_tags, decode_html
from ox.text import find_re

from . import imdb

def get_id(url):
    return url.split("/")[-1]

def get_url(id):
    return "https://www.criterion.com/films/%s" % id

def get_data(id, timeout=ox.cache.cache_timeout, get_imdb=False):
    '''
    >>> get_data('1333').get('imdbId')
    '0060304'

    >>> get_data('236')['posters'][0]
    'http://s3.amazonaws.com/criterion-production/release_images/1586/ThirdManReplace.jpg'

    >>> get_data('786')['posters'][0]
    'http://s3.amazonaws.com/criterion-production/product_images/185/343_box_348x490.jpg'
    '''
    data = {
        "id": id,
        "url": get_url(id)
    }
    try:
        html = read_url(data["url"], timeout=timeout, unicode=True)
    except:
        html = read_url(data["url"], timeout=timeout).decode('utf-8', 'ignore')

    data["number"] = find_re(html, "<b>Spine #(\d+)")

    data["title"] = decode_html(find_re(html, "<h1 class=\"header__primarytitle\".*?>(.*?)</h1>"))
    data["title"] = data["title"].split(' \u2014 The Television Version')[0].strip()
    results = find_re(html, '<ul class="film-meta-list">(.*?)</ul>')
    info = re.compile('<li itemprop="(.*?)".*?>(.*?)</li>', re.DOTALL).findall(results)
    info = {k: strip_tags(v).strip() for k, v in info}
    meta = re.compile('<meta.*? name="(.*?)".*? content="(.*?)"', re.DOTALL).findall(html)
    meta = {k: v.strip() for k, v in meta}
    if 'director' in info:
        data['director'] = info['director']
    elif 'director' in meta:
        data['director'] = meta['director']
    if 'countryOfOrigin' in info:
        data['country'] = [c.strip() for c in decode_html(info['countryOfOrigin']).split(', ')]
    if 'inLanguage' in info:
        data['language'] = [l.strip() for l in decode_html(info['inLanguage']).split(', ')]
    for v in re.compile('<li>(.*?)</li>', re.DOTALL).findall(results):
        if 'datePublished' in v:
            data['year'] = strip_tags(v).strip()
        elif 'duration' in v:
            data['duration'] = strip_tags(v).strip()
    data["synopsis"] = decode_html(strip_tags(find_re(html,
                                   "<div class=\"product-summary\".*?>.*?<p>(.*?)</p>")))

    result = find_re(html, "<div class=\"purchase\">(.*?)</div>")
    if 'Blu-Ray' in result or 'Essential Art House DVD' in result:
        r = re.compile('<h3 class="section_title first">Other Editions</h3>(.*?)</div>', re.DOTALL).findall(html)
        if r:
            result = r[0]
    result = find_re(result, "<a href=\"(.*?)\"")
    if not "/boxsets/" in result:
        data["posters"] = [result]
    else:
        html_ = read_url(result, unicode=True)
        result = find_re(html_, '//www.criterion.com/films/%s.*?">(.*?)</a>' % id)
        result = find_re(result, "src=\"(.*?)\"")
        if result:
            data["posters"] = [result.replace("_w100", "")]
        else:
            data["posters"] = []
    data['posters'] = [re.sub('(\?\d+)$', '', p) for p in data['posters']]
    data['posters'] = [p for p in data['posters'] if p]

    posters = find_re(html, '<div class="product-box-art".*?>(.*?)</div>')
    for poster in re.compile('<img src="(.*?)"').findall(posters):
        data['posters'].append(poster)

    result = re.compile('<div class="gallery-item ">.*?src="(.*?)"', re.DOTALL).findall(html)
    #result = find_re(html, "<img alt=\"Film Still\" height=\"252\" src=\"(.*?)\"")
    if result:
        data["stills"] = [result]
        data["trailers"] = []
    else:
        data["stills"] = list(filter(lambda x: x, [find_re(html, "\"thumbnailURL\", \"(.*?)\"")]))
        data["trailers"] = list(filter(lambda x: x, [find_re(html, "\"videoURL\", \"(.*?)\"")]))

    if timeout == ox.cache.cache_timeout:
        timeout = -1
    if get_imdb and 'title' in data and 'director' in data:
        # removed year, as "title (year)" may fail to match
        data['imdbId'] = imdb.get_movie_id(data['title'], data['director'], timeout=timeout)
    return data

def get_ids(page=None):
    ids = []
    html = read_url("https://www.criterion.com/shop/browse/list?sort=spine_number", unicode=True)
    results = re.compile("films/(\d+)-").findall(html)
    ids += results
    results = re.compile("boxsets/(.*?)\"").findall(html)
    for result in results:
        html = read_url("https://www.criterion.com/boxsets/" + result, unicode=True)
        results = re.compile("films/(\d+)-").findall(html)
        ids += results
    return sorted(set(ids), key=int)


if __name__ == '__main__':
    print(get_ids())



# -*- coding: utf-8 -*-
# vi:si:et:sw=4:sts=4:ts=4
from __future__ import print_function

import re

import urllib

from ox.utils import json
from ox.cache import read_url
from ox import find_re


def get_id(url):
    return url.split("/")[-1]

def get_url(id=None, imdb=None, allmovie=None):
    if imdb:
        query = '"%s"' % imdb
        result = find(query)
        if result:
            url = result[0][1]
            data = get_movie_data(url)
            if 'imdb_id' in data:
                return url
        return ""
    if allmovie:
        query = '"amg_id = 1:%s"' % allmovie
        result = find(query)
        if result:
            url = result[0][1]
            return url
        return ''
    return "http://en.wikipedia.org/wiki/%s" % id

def get_movie_id(title, director='', year=''):
    query = '"%s" film %s %s' % (title, director, year)
    result = find(query, 1)
    if result:
        return result[0][1]
    return ''

def get_wiki_data(wikipedia_url):
    url = wikipedia_url.replace('wikipedia.org/wiki/', 'wikipedia.org/w/index.php?title=')
    url = "%s&action=raw" % url
    data = read_url(url).decode('utf-8')
    return data

def get_movie_data(wikipedia_url):
    if not wikipedia_url.startswith('http'):
        wikipedia_url = get_url(wikipedia_url)
    data = get_wiki_data(wikipedia_url)
    filmbox_data = find_re(data, '''\{\{[Ii]nfobox.[Ff]ilm(.*?)\n\}\}''')
    filmbox = {}
    _box = filmbox_data.strip().split('|')
    for row in _box:
        d = row.split('=')
        if len(d) == 2:
            _key = d[0].strip()
            if _key:
                key = _key
                if key[0] == '|':
                    key = key[1:]
                key = key.strip()
                value = d[1].strip()
                value = value.replace('<!-- see WP:ALT -->', '')
                if '<br>' in value:
                    value = value.split('<br>')
                if value:
                    if key in filmbox:
                        if isinstance(value, list) and isinstance(filmbox[key], str):
                            filmbox[key] = [filmbox[key]] + value
                        else:
                            filmbox[key] += value
                        if isinstance(filmbox[key], list):
                            filmbox[key] = [k for k in filmbox[key] if k]
                    else:
                        filmbox[key] = value
    if not filmbox_data:
        return filmbox
    if 'amg_id' in filmbox and not filmbox['amg_id'].isdigit():
        del filmbox['amg_id']
    if 'Allmovie movie' in data:
        filmbox['amg_id'] = find_re(data, 'Allmovie movie\|.*?(\d+)')
    elif 'Allmovie title' in data:
        filmbox['amg_id'] = find_re(data, 'Allmovie title\|.*?(\d+)')

    if 'Official website' in data:
        filmbox['website'] = find_re(data, 'Official website\|(.*?)}').strip()

    r = re.compile('{{IMDb title\|id=(\d{7})', re.IGNORECASE).findall(data)
    if r:
        filmbox['imdb_id'] = r[0]
    else:
        r = re.compile('{{IMDb title\|(\d{7})', re.IGNORECASE).findall(data)
        if r:
            filmbox['imdb_id'] = r[0]

    r = re.compile('{{Internet Archive.*?\|id=(.*?)[\|}]', re.IGNORECASE).findall(data)
    if r:
        filmbox['archiveorg_id'] = r[0]

    r = re.compile('{{mojo title\|(.*?)[\|}]', re.IGNORECASE).findall(data)
    if r:
        filmbox['mojo_id'] = r[0].replace('id=', '')

    r = re.compile('{{rotten-tomatoes\|(.*?)[\|}]', re.IGNORECASE).findall(data)
    if r:
        filmbox['rottentomatoes_id'] = r[0].replace('id=', '')
    if 'google video' in data:
        filmbox['google_video_id'] = find_re(data, 'google video\|.*?(\d*?)[\|}]')
    if 'DEFAULTSORT' in data:
        filmbox['title_sort'] = find_re(data, '''\{\{DEFAULTSORT:(.*?)\}\}''')
    return filmbox

def get_image_url(name):
    url = 'http://en.wikipedia.org/wiki/Image:' + name.replace(' ', '%20')
    data = read_url(url).decode('utf-8')
    url = find_re(data, 'href="(http://upload.wikimedia.org/.*?)"')
    if not url:
        url = find_re(data, 'href="(//upload.wikimedia.org/.*?)"')
        if url:
            url = 'http:' + url
    return url

def get_poster_url(wikipedia_url):
    if not wikipedia_url.startswith('http'): wikipedia_url = get_url(wikipedia_url)
    data = get_movie_data(wikipedia_url)
    if 'image' in data:
        return get_image_url(data['image'])
    return ''

def get_movie_poster(wikipedia_url):
    # deprecated, use get_poster_url()
    return get_poster_url(wikipedia_url)

def get_allmovie_id(wikipedia_url):
    data = get_movie_data(wikipedia_url)
    return data.get('amg_id', '')

def find(query, max_results=10):
    query = {'action': 'query', 'list': 'search', 'format': 'json',
             'srlimit': max_results, 'srwhat': 'text', 'srsearch': query.encode('utf-8')}
    url = "http://en.wikipedia.org/w/api.php?" + urllib.parse.urlencode(query)
    data = read_url(url)
    if not data:
        data = read_url(url, timeout=0)
    result = json.loads(data.decode('utf-8'))
    results = []
    if result and 'query' in result:
        for r in result['query']['search']:
            title = r['title']
            url = "http://en.wikipedia.org/wiki/%s" % title.replace(' ', '_')
            results.append((title, url, ''))
    return results




# -*- coding: utf-8 -*-
# vi:si:et:sw=4:sts=4:ts=4
# GPL 2008
import re
import unicodedata


_articles = ('the', 'la', 'a', 'die', 'der', 'le', 'el',
             "l'", 'il', 'das', 'les', 'o', 'ein', 'i', 'un', 'los', 'de',
             'an', 'una', 'las', 'eine', 'den', 'gli', 'het', 'os', 'lo',
             'az', 'det', 'ha-', 'een', 'ang', 'oi', 'ta', 'al-', 'dem',
             'mga', 'uno', "un'", 'ett', u'\xcf', 'eines', u'\xc7', 'els',
             u'\xd4\xef', u'\xcf\xe9')

# Articles in a dictionary.
_articlesDict = dict([(x, x) for x in _articles])
_spArticles = []
for article in _articles:
    if article[-1] not in ("'", '-'):
        article += ' '
    _spArticles.append(article)

_noarticles = (
    'los angeles',
    'i am ',
    'i be area',
    'i call ',
    'i come ',
    'i confess',
    'i hired ',
    'i killed ',
    'i know ',
    'i live ',
    'i love',
    'i married',
    'i never',
    'i shot',
    'i start',
    'i was',
)

def canonical_title(title):
    """Return the title in the canonic format 'Movie Title, The'.
    
    >>> canonical_title('The Movie Title')
    'Movie Title, The'

    >>> canonical_title('Los Angeles Plays Itself')
    'Los Angeles Plays Itself'
    """
    try:
        if title.split(', ')[-1].lower() in _articlesDict:
            return title
    except IndexError:
        pass
    ltitle = title.lower()
    for start in _noarticles:
        if ltitle.startswith(start):
            return title
    for article in _spArticles:
        if ltitle.startswith(article):
            lart = len(article)
            title = '%s, %s' % (title[lart:], title[:lart])
            if article[-1] == ' ':
                title = title[:-1]
            break
    ## XXX: an attempt using a dictionary lookup.
    ##for artSeparator in (' ', "'", '-'):
    ##    article = _articlesDict.get(ltitle.split(artSeparator)[0])
    ##    if article is not None:
    ##        lart = len(article)
    ##        # check titles like "una", "I'm Mad" and "L'abbacchio".
    ##        if title[lart:] == '' or (artSeparator != ' ' and
    ##                                title[lart:][1] != artSeparator): continue
    ##        title = '%s, %s' % (title[lart:], title[:lart])
    ##        if artSeparator == ' ': title = title[1:]
    ##        break
    return title

def normalize_title(title):
    """Return the title in the normal "The Title" format.

    >>> normalize_title('Movie Title, The')
    'The Movie Title'
    """
    stitle = title.split(', ')
    if len(stitle) > 1 and stitle[-1].lower() in _articlesDict:
        sep = ' '
        if stitle[-1][-1] in ("'", '-'):
            sep = ''
        title = '%s%s%s' % (stitle[-1], sep, ', '.join(stitle[:-1]))
    return title

def normalize_imdbid(imdbId):
    """Return 7 digit imdbId.

    >>> normalize_imdbid('http://www.imdb.com/title/tt0159206/')
    '0159206'
    >>> normalize_imdbid(159206)
    '0159206'
    >>> normalize_imdbid('tt0159206')
    '0159206'
    """
    if isinstance(imdbId, str):
        imdbId = re.sub('.*(\d{7}).*', '\\1', imdbId)
    elif isinstance(imdbId, int):
        imdbId = "%07d" % imdbId
    return imdbId


# Common suffixes in surnames.
_sname_suffixes = (
    'al', 'ben', 'da', 'de', 'del', 'den', 'der', 'des', 'di', 'dos', 'du',
    'e', 'el', 'la', 'le', 'the', 'vom', 'von', 'van', 'y'
)

def canonical_name(name):
    """Return the given name in canonical "Surname, Name" format.
    It assumes that name is in the 'Name Surname' format.
    
    >>> canonical_name('Jean Luc Godard')
    'Godard, Jean Luc'

    >>> canonical_name('Ivan Ivanov-Vano')
    'Ivanov-Vano, Ivan'

    >>> canonical_name('Gus Van Sant')
    'Van Sant, Gus'

    >>> canonical_name('Brian De Palma')
    'De Palma, Brian'
    """

    # XXX: some statistics (over 1852406 names):
    #      - just a surname:                 51921
    #      - single surname, single name:  1792759
    #      - composed surname, composed name: 7726
    #      - composed surname, single name:  55623
    #        (2: 49259, 3: 5502, 4: 551)
    #      - single surname, composed name: 186604
    #        (2: 178315, 3: 6573, 4: 1219, 5: 352)
    # Don't convert names already in the canonical format.
    if name in ('Unknown Director', ):
        return name
    if name.find(', ') != -1:
        return name
    sname = name.split(' ')
    snl = len(sname)
    if snl == 2:
        # Just a name and a surname: how boring...
        name = '%s, %s' % (sname[1], sname[0])
    elif snl > 2:
        lsname = [x.lower() for x in sname]
        if snl == 3:
            _indexes = (0, snl-2)
        else:
            _indexes = (0, snl-2, snl-3)
        # Check for common surname prefixes at the beginning and near the end.
        for index in _indexes:
            if lsname[index] not in _sname_suffixes:
                continue
            try:
                # Build the surname.
                surn = '%s %s' % (sname[index], sname[index+1])
                del sname[index]
                del sname[index]
                try:
                    # Handle the "Jr." after the name.
                    if lsname[index+2].startswith('jr'):
                        surn += ' %s' % sname[index]
                        del sname[index]
                except (IndexError, ValueError):
                    pass
                name = '%s, %s' % (surn, ' '.join(sname))
                break
            except ValueError:
                continue
        else:
            name = '%s, %s' % (sname[-1], ' '.join(sname[:-1]))
    return name

def normalize_name(name):
    """Return a name in the normal "Name Surname" format.
    
    >>> normalize_name('Godard, Jean Luc')
    'Jean Luc Godard'

    >>> normalize_name('Ivanov-Vano, Ivan')
    'Ivan Ivanov-Vano'

    >>> normalize_name('Van Sant, Gus')
    'Gus Van Sant'

    >>> normalize_name('De Palma, Brian')
    'Brian De Palma'
    """
    sname = name.split(', ')
    if len(sname) == 2:
        name = '%s %s' % (sname[1], sname[0])
    return name

def normalize_path(path):
    path = path.replace(':', '_').replace('/', '_')
    if path.endswith('.'):
        path = path[:-1] + '_'
    return path

def strip_accents(s):
    if isinstance(s, str):
        s = s.decode('utf-8')
    return ''.join((c for c in unicodedata.normalize('NFD', s) if unicodedata.category(c) != 'Mn'))




# -*- coding: utf-8 -*-
# vi:si:et:sw=4:sts=4:ts=4
from __future__ import print_function
import re

import lxml.html

from ox import strip_tags, decode_html
from ox.cache import read_url


def get_id(url):
    return url.replace('http://www.ubu.com/', '').split('.html')[0].replace('/./', '/')

def get_url(id):
    return 'http://www.ubu.com/%s.html' % id

def get_data(url):
    if not url.startswith('http:'):
        url = get_url(url)
    data = read_url(url, unicode=True)
    m = {
        'id': get_id(url),
        'url': url,
        'type': re.compile('ubu.com/(.*?)/').findall(url)[0]
    }
    if m['type'] == 'sound':
        m['tracks'] = [{
            'title': strip_tags(decode_html(t[1])).strip(),
            'url': t[0]
        } for t in re.compile('"(http.*?.mp3)"[^>]*>(.+)</a', re.IGNORECASE).findall(data)]
    else:
        for videourl, title in re.compile('href="(http://ubumexico.centro.org.mx/.*?)">(.*?)</a>').findall(data):
            if videourl.endswith('.srt'):
                m['srt'] = videourl
            elif not 'video' in m:
                m['video'] = videourl
                m['video'] = m['video'].replace('/video/ ', '/video/').replace(' ', '%20')
                if m['video'] == 'http://ubumexico.centro.org.mx/video/':
                    del m['video']
            if not 'title' in m:
                m['title'] = strip_tags(decode_html(title)).strip()
        if not 'url' in m:
            print(url, 'missing')
        if 'title' in m:
            m['title'] = re.sub('(.*?) \(\d{4}\)$', '\\1', m['title'])

        if not 'title' in m:
            match = re.compile('<span id="ubuwork">(.*?)</span>').findall(data)
            if match:
                m['title'] = strip_tags(decode_html(match[0])).strip()
        if not 'title' in m:
            match = re.compile("<title>.*?&amp;(.*?)</title>", re.DOTALL).findall(data)
            if match:
                m['title'] = re.sub('\s+', ' ', match[0]).strip()
                if ' - ' in m['title']:
                    m['title'] = m['title'].split(' - ', 1)[-1]
        if 'title' in m:
            m['title'] = strip_tags(decode_html(m['title']).strip())
        match = re.compile("flashvars','file=(.*?.flv)'").findall(data)
        if match:
            m['flv'] = match[0]
            m['flv'] = m['flv'].replace('/video/ ', '/video/').replace(' ', '%20')

        match = re.compile('''src=(.*?) type="video/mp4"''').findall(data)
        if match:
            m['mp4'] = match[0].strip('"').strip("'").replace(' ', '%20')
            if not m['mp4'].startswith('http'):
                m['mp4'] = 'http://ubumexico.centro.org.mx/video/' + m['mp4']
        elif 'video' in m and (m['video'].endswith('.mp4') or m['video'].endswith('.m4v')):
            m['mp4'] = m['video']

        doc = lxml.html.document_fromstring(read_url(url))
        desc = doc.xpath("//div[contains(@id, 'ubudesc')]")
        if len(desc):
            txt = []
            for part in desc[0].text_content().split('\n\n'):
                if part == 'RESOURCES:':
                    break
                if part.strip():
                    txt.append(part)
            if txt:
                if len(txt) > 1 and txt[0].strip() == m.get('title'):
                    txt = txt[1:]
                m['description'] = '\n\n'.join(txt).split('RESOURCES')[0].split('RELATED')[0].strip()
        y = re.compile('\((\d{4})\)').findall(data)
        if y:
            m['year'] = int(y[0])
        d = re.compile('Director: (.+)').findall(data)
        if d:
            m['director'] = strip_tags(decode_html(d[0])).strip()

        a = re.compile('<a href="(.*?)">Back to (.*?)</a>', re.DOTALL).findall(data)
        if a:
            m['artist'] = strip_tags(decode_html(a[0][1])).strip()
        else:
            a = re.compile('<a href="(.*?)">(.*?) in UbuWeb Film').findall(data)
            if a:
                m['artist'] = strip_tags(decode_html(a[0][1])).strip()
            else:
                a = re.compile('<b>(.*?)\(b\..*?\d{4}\)').findall(data)
                if a:
                    m['artist'] = strip_tags(decode_html(a[0])).strip()
                elif m['id'] == 'film/lawder_color':
                    m['artist'] = 'Standish Lawder'

        if 'artist' in m:
            m['artist'] = m['artist'].replace('in UbuWeb Film', '')
            m['artist'] = m['artist'].replace('on UbuWeb Film', '').strip()
        if m['id'] == 'film/coulibeuf':
            m['title'] = 'Balkan Baroque'
            m['year'] = 1999
    return m

def get_films():
    ids = get_ids()
    films = []
    for id in ids:
        info = get_data(id)
        if info['type'] == 'film' and ('flv' in info or 'video' in info):
            films.append(info)
    return films

def get_ids():
    data = read_url('http://www.ubu.com/film/')
    ids = []
    author_urls = []
    for url, author in re.compile('<a href="(\./.*?)">(.*?)</a>').findall(data):
        url = 'http://www.ubu.com/film' + url[1:]
        data = read_url(url)
        author_urls.append(url)
        for u, title in re.compile('<a href="(.*?)">(.*?)</a>').findall(data):
            if not u.startswith('http'):
                if u == '../../sound/burroughs.html':
                    u = 'http://www.ubu.com/sound/burroughs.html'
                elif u.startswith('../'):
                    u = 'http://www.ubu.com/' + u[3:]
                else:
                    u = 'http://www.ubu.com/film/' + u
                if u not in author_urls and u.endswith('.html'):
                    ids.append(u)
    ids = [get_id(url) for url in list(set(ids))]
    return ids

def get_sound_ids():
    data = read_url('http://www.ubu.com/sound/')
    ids = []
    for url, author in re.compile('<a href="(\./.*?)">(.*?)</a>').findall(data):
        url = 'http://www.ubu.com/sound' + url[1:]
        ids.append(url)
    ids = [get_id(url) for url in sorted(set(ids))]
    return ids



# -*- coding: utf-8 -*-
# vi:si:et:sw=4:sts=4:ts=4

import json

from ox.cache import read_url


def get_data(id):
    url = 'http://vimeo.com/api/v2/video/%s.json' % id
    data = json.loads(read_url(url).decode('utf-8'))[0]

    url = 'http://player.vimeo.com/video/%s/config?autoplay=0&byline=0&bypass_privacy=1&context=clip.main&default_to_hd=1&portrait=0' % id    
    info = json.loads(read_url(url).decode('utf-8'))
    data['video'] = info['request']['files']['h264']
    return data




# -*- coding: utf-8 -*-
# vi:si:et:sw=4:sts=4:ts=4
import ox.cache

def get_poster_url(id):
    url = "http://0xdb.org/%s/poster.0xdb.jpg" % id
    if ox.cache.exists(url):
        return url
    return ''




# -*- coding: utf-8 -*-
# vi:si:et:sw=4:sts=4:ts=4
# GPL 2009
from __future__ import print_function
import os

from ox.utils import json

def get(key):
    user_auth = os.environ.get('oxAUTH', os.path.expanduser('~/.ox/auth.json'))
    auth = {}
    if os.path.exists(user_auth):
        f = open(user_auth, "r")
        data = f.read()
        f.close()
        auth = json.loads(data)
    if key in auth:
        return auth[key]
    print("please add key %s to json file '%s'" % (key, user_auth))
    raise Exception("no key %s found" % key)

def update(key, value):
    user_auth = os.environ.get('oxAUTH', os.path.expanduser('~/.ox/auth.json'))
    auth = {}
    if os.path.exists(user_auth):
        f = open(user_auth, "r")
        data = f.read()
        f.close()
        auth = json.loads(data)
    auth[key] = value
    f = open(user_auth, "w")
    f.write(json.dumps(auth, indent=2))
    f.close()



# -*- coding: utf-8 -*-
# vi:si:et:sw=4:sts=4:ts=4
from urllib.parse import quote, unquote_plus
import urllib
from http import cookiejar as cookielib
import re
from xml.dom.minidom import parseString
import json

import ox
from ox.cache import read_url, cache_timeout


def get_id(url):
    match = re.compile('v=(.+?)($|&)').findall(url)
    if match:
        return match[0][0]

def get_url(id):
    return 'http://www.youtube.com/watch?v=%s' % id

def video_url(youtubeId, format='mp4', timeout=cache_timeout):
    """
        youtubeId - if of video
        format - video format, options: webm, 1080p, 720p, mp4, high
    """
    fmt = None
    if format == '4k':
        fmt = 38
    elif format == '1080p':
        fmt = 37
    elif format == '720p':
        fmt = 22
    elif format == 'mp4':
        fmt = 18
    elif format == 'high':
        fmt = 35
    elif format == 'webm':
        streams = videos(youtubeId, 'webm')
        return streams[max(streams.keys())]['url']

    streams = videos(youtubeId)
    if str(fmt) in streams:
        return streams[str(fmt)]['url']

def get_video_info(id):
    eurl = get_url(id)
    data = read_url(eurl).decode('utf-8')
    t = re.compile('\W[\'"]?t[\'"]?: ?[\'"](.+?)[\'"]').findall(data)
    if t:
        t = t[0]
    else:
        raise IOError
    url = "http://www.youtube.com/get_video_info?&video_id=%s&el=$el&ps=default&eurl=%s&hl=en_US&t=%s" % (id, quote(eurl), quote(t))
    data = read_url(url).decode('utf-8')
    info = {}
    for part in data.split('&'):
        key, value = part.split('=')
        info[key] = unquote_plus(value).replace('+', ' ')
    return info

def find(query, max_results=10, offset=1, orderBy='relevance'):
    import feedparser
    query = quote(query)
    url = "http://gdata.youtube.com/feeds/api/videos?vq=%s&orderby=%s&start-index=%s&max-results=%s" % (query, orderBy, offset, max_results)
    data = read_url(url)
    fd = feedparser.parse(data)
    videos = []
    for item in fd.entries:
        id = item['id'].split('/')[-1]
        title = item['title']
        description = item['description']
        videos.append((title, id, description))
        if len(videos) >= max_results:
            return videos
    return videos

def info(id, timeout=cache_timeout):
    info = {}
    if id.startswith('http'):
        id = get_id(id)
        if not id:
            return info
    url = "http://gdata.youtube.com/feeds/api/videos/%s?v=2" % id
    data = read_url(url, timeout=timeout)
    xml = parseString(data)
    info['id'] = id
    info['url'] = get_url(id)
    info['title'] = xml.getElementsByTagName('title')[0].firstChild.data
    info['description'] = xml.getElementsByTagName('media:description')[0].firstChild.data
    info['date'] = xml.getElementsByTagName('published')[0].firstChild.data.split('T')[0]
    info['author'] = "http://www.youtube.com/user/%s"%xml.getElementsByTagName('name')[0].firstChild.data

    info['categories'] = []
    for cat in xml.getElementsByTagName('media:category'):
        info['categories'].append(cat.firstChild.data)

    k = xml.getElementsByTagName('media:keywords')[0].firstChild
    if k:
        info['keywords'] = k.data.split(', ')
    data = read_url(info['url'], timeout=timeout)
    match = re.compile('<h4>License:</h4>(.*?)</p>', re.DOTALL).findall(data)
    if match:
        info['license'] = match[0].strip()
        info['license'] = re.sub('<.+?>', '', info['license']).strip()

    subs = subtitles(id, timeout)
    if subs:
        info['subtitles'] = subs
    return info

def subtitles(id, timeout=cache_timeout):
    url = "http://www.youtube.com/api/timedtext?hl=en&type=list&tlangs=1&v=%s&asrs=1" % id
    data = read_url(url, timeout=timeout)
    xml = parseString(data)
    languages = [t.getAttribute('lang_code') for t in xml.getElementsByTagName('track')]
    subtitles = {}
    if languages:
        for language in languages:
            url = "http://www.youtube.com/api/timedtext?hl=en&v=%s&type=track&lang=%s&name&kind" % (id, language)
            data = read_url(url, timeout=timeout)
            xml = parseString(data)
            subs = []
            for t in xml.getElementsByTagName('text'):
                start = float(t.getAttribute('start'))
                duration = t.getAttribute('dur')
                if not duration:
                    duration = '2'
                end = start + float(duration)
                if t.firstChild:
                    text = t.firstChild.data
                    subs.append({
                        'in': start,
                        'out': end,
                        'value': ox.decode_html(text),
                    })
            subtitles[language] = subs
    return subtitles

def videos(id, format=''):
    stream_type = {
        'flv': 'video/x-flv',
        'webm': 'video/webm',
        'mp4': 'video/mp4'
    }.get(format)
    info = get_video_info(id)
    stream_map = info['url_encoded_fmt_stream_map']
    streams = {}
    for x in stream_map.split(','):
        stream = {}
        #for s in x.split('\\u0026'):
        for s in x.split('&'):
            key, value = s.split('=')
            value = unquote_plus(value)
            stream[key] = value
        if 'url' in stream and 'sig' in stream:
            stream['url'] = '%s&signature=%s' % (stream['url'], stream['sig'])
        if not stream_type or stream['type'].startswith(stream_type):
            streams[stream['itag']] = stream
    return streams

def playlist(url):
    data = read_url(url).decode('utf-8')
    items = []
    for i in list(set(re.compile('<a href="(/watch\?v=.*?)" title="(.*?)" ').findall(data))):
        items.append({
            'title': i[1],
            'url': 'http://www.youtube.com' + i[0].split('&amp;')[0]
        })
    return items

def download_webm(id, filename):
    stream_type = 'video/webm'
    url = "http://www.youtube.com/watch?v=%s" % id
    cj = cookielib.CookieJar()
    opener = urllib.request.build_opener(urllib.request.HTTPCookieProcessor(cj))
    opener.addheaders = [
        ('User-Agent',
         'Mozilla/5.0 (X11; Linux i686; rv:2.0) Gecko/20100101 Firefox/4.0'),
        ('Accept-Language', 'en-us, en;q=0.50')
    ]
    u = opener.open(url)
    data = u.read()
    u.close()
    match = re.compile('"url_encoded_fmt_stream_map": "(.*?)"').findall(data)
    streams = {}
    for x in match[0].split(','):
        stream = {}
        for s in x.split('\\u0026'):
            key, value = s.split('=')
            value = unquote_plus(value)
            stream[key] = value
        if stream['type'].startswith(stream_type):
            streams[stream['itag']] = stream
    if streams:
        s = max(streams.keys())
        url = streams[s]['url']
        if 'sig' in streams[s]:
            url += 'signature=' + streams[s]['sig']
    else:
        return None

    #download video and save to file.
    u = opener.open(url)
    f = open(filename, 'w')
    data = True
    while data:
        data = u.read(4096)
        f.write(data)
    f.close()
    u.close()
    return filename

def get_config(id):
    if id.startswith('http'):
        url = id
    else:
        url = get_url(id)
    data = read_url(url)
    match = re.compile('ytplayer.config = (.*?);<').findall(data)
    if match:
        config = json.load(match[0])
    return config
